@@ -11,8 +11,8 @@
  *     Mikael Lagerkvist, 2010
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-09-19 14:02:26 +0200 (Mon, 19 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12400 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -121,20 +121,20 @@ public:
 
   /// Actual model
   BACP(const SizeOptions& opt) : curr(curriculum[opt.size()]) {
+    std::map<std::string, int> courseMap; // Map names to course numbers
+    int maxCredit = 0;
+    int numberOfCourses = 0;
+    for (const Course* co=curr.courses; co->name != 0; co++) {
+      courseMap[co->name] = numberOfCourses++;
+      maxCredit += co->credit;
+    }
+
     int p = curr.p;
     int a = curr.a;
     int b = curr.b;
     int c = curr.c;
     int d = curr.d;
 
-    std::map<std::string, int> courseMap; // Map names to course numbers
-    int maxCredit = 0;
-    int numberOfCourses = 0;
-    for (const Course* c=curr.courses; c->name != 0; c++) {
-      courseMap[c->name] = numberOfCourses++;
-      maxCredit += c->credit;
-    }
-
     l = IntVarArray(*this, p, a, b);
     u = IntVar(*this, 0, maxCredit);
     q = IntVarArray(*this, p, c, d);
@@ -154,9 +154,8 @@ public:
     }
 
     // Precedence
-    for (const char** prereq = curr.prereqs; *prereq != 0; prereq+=2) {
+    for (const char** prereq = curr.prereqs; *prereq != 0; prereq+=2)
       rel(*this, x[courseMap[*prereq]] < x[courseMap[*(prereq+1)]]);
-    }
 
     // Optimization criterion: minimize u
     max(*this, l, u);
