@@ -7,8 +7,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-07-13 12:31:25 +0200 (Wed, 13 Jul 2011) $ by $Author: schulte $
+ *     $Revision: 12182 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -128,33 +128,23 @@ public:
 
       // Symmetry breaking: order groups
       for (int j=0; j<w; j++) {
-        for (int i=0; i<g-1; i++) {
-          rel(*this, min(schedule(i,j)) < min(schedule(i+1,j)));
-        }
+        IntVarArgs m(g);
+        for (int i=0; i<g; i++)
+          m[i] = expr(*this, min(schedule(i,j)));
+        rel(*this, m, IRT_LE);
       }
 
       // Symmetry breaking: order weeks
       // minElem(group(w,0)\{0}) < minElem(group(w+1,0)\{0})
-      for (int i=0; i<w-1; i++) {
-        rel(*this, min(schedule(0,i)-IntSet(0,0)) <
-                   min(schedule(0,i+1)-IntSet(0,0)));
-        
+      {
+        IntVarArgs m(w);
+        for (int i=0; i<w; i++)
+          m[i] = expr(*this, min(schedule(0,i)-IntSet(0,0)));
+        rel(*this, m, IRT_LE);
       }
 
-      // Initialize the dual variables:
-      // gInv(i,p) is player p's group in week i
-      Matrix<IntVarArgs> gInv(IntVarArgs(*this, w*g*s, 0, g-1),w,g*s);
-     
-      for (int i=0; i<w; i++)
-        for (int p=0; p<g*s; p++) {
-          SetVar player(*this, p,p, 0, g*s-1);
-          element(*this, schedule.row(i), gInv(i,p),player);
-        }
-      
-      // Symmetry breaking: order players
-      for (int i=0; i<w; i++)
-        for (int p=0; p<g; p++)
-          rel(*this, gInv(i,p), IRT_LQ, p);
+      // Symmetry breaking: value symmetry of player numbers
+      precede(*this, groups, IntArgs::create(groups.size(),0));
     }
 
     branch(*this, groups, SET_VAR_MIN_MIN, SET_VAL_MIN_INC);
