@@ -9,8 +9,8 @@
  *     Christian Schulte, 2001
  *
  *  Last modified:
- *     $Date: 2011-02-01 14:31:23 +0100 (Tue, 01 Feb 2011) $ by $Author: schulte $
- *     $Revision: 11595 $
+ *     $Date: 2011-05-25 16:56:41 +0200 (Wed, 25 May 2011) $ by $Author: schulte $
+ *     $Revision: 12022 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -40,9 +40,8 @@
 #include <gecode/driver.hh>
 #include <gecode/int.hh>
 #include <gecode/minimodel.hh>
-#include <gecode/graph.hh>
 
-#ifdef GECODE_HAS_GIST
+#if defined(GECODE_HAS_QT) && defined(GECODE_HAS_GIST)
 #include <QtGui>
 #endif
 
@@ -79,6 +78,11 @@ protected:
     virtual size_t size(void) const {
       return sizeof(Choice);
     }
+    /// Archive into \a e
+    virtual void archive(Archive& e) const {
+      Gecode::Choice::archive(e);
+      e << pos << val;
+    }
   };
  
   /// Construct brancher
@@ -117,6 +121,12 @@ public:
     }
     return new Choice(*this, start, n);
   }
+  /// Return choice
+  virtual Choice* choice(const Space&, Archive& e) {
+    int pos, val;
+    e >> pos >> val;
+    return new Choice(*this, pos, val);
+  }
   /// Perform commit for choice \a _c and alternative \a a
   virtual ExecStatus commit(Space& home, const Gecode::Choice& _c, 
                             unsigned int a) {
@@ -312,7 +322,7 @@ public:
  *
  */
 
-#ifdef GECODE_HAS_GIST
+#if defined(GECODE_HAS_QT) && defined(GECODE_HAS_GIST)
 /// Inspector showing knight moves on a chess board
 class KnightsInspector : public Gist::Inspector {
 protected:
@@ -401,7 +411,7 @@ main(int argc, char* argv[]) {
   opt.branching(Knights::BRANCH_NAIVE, "reified");
   opt.branching(Knights::BRANCH_WARNSDORFF, "warnsdorff");
 
-#ifdef GECODE_HAS_GIST
+#if defined(GECODE_HAS_QT) && defined(GECODE_HAS_GIST)
   KnightsInspector ki;
   opt.inspect.click(&ki);
 #endif
