@@ -7,8 +7,8 @@
  *     Mikael Lagerkvist, 2006
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -184,6 +184,11 @@ public:
       virtual size_t size(void) const {
         return sizeof(Choice);
       }
+      /// Archive into \a e
+      virtual void archive(Archive& e) const {
+        Gecode::Choice::archive(e);
+        e << pos << val;
+      }
     };
 
     /// Construct brancher
@@ -226,6 +231,12 @@ public:
       assert(pos != -1);
       return new Choice(*this, pos, true);
     }
+    /// Return choice
+    virtual Choice* choice(const Space&, Archive& e) {
+      int pos, val;
+      e >> pos >> val;
+      return new Choice(*this, pos, val);
+    }
     /** \brief Perform commit for choice \a _c and
      * alternative \a a.
      */
