@@ -7,8 +7,8 @@
  *     Christian Schulte, 2001
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-05-02 16:31:59 +0200 (Mon, 02 May 2011) $ by $Author: schulte $
+ *     $Revision: 11982 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -39,7 +39,7 @@
 #include <gecode/int.hh>
 #include <gecode/minimodel.hh>
 
-#ifdef GECODE_HAS_GIST
+#if defined(GECODE_HAS_QT) && defined(GECODE_HAS_GIST)
 #include <QtGui>
 #endif
 
@@ -118,7 +118,7 @@ public:
   }
 };
 
-#ifdef GECODE_HAS_GIST
+#if defined(GECODE_HAS_QT) && defined(GECODE_HAS_GIST)
 /// Inspector showing queens on a chess board
 class QueensInspector : public Gist::Inspector {
 protected:
@@ -200,7 +200,7 @@ main(int argc, char* argv[]) {
   opt.propagation(Queens::PROP_DISTINCT, "distinct",
                       "three distinct constraints");
 
-#ifdef GECODE_HAS_GIST
+#if defined(GECODE_HAS_QT) && defined(GECODE_HAS_GIST)
   QueensInspector ki;
   opt.inspect.click(&ki);
 #endif
