@@ -11,8 +11,8 @@
  *     Mikael Lagerkvist, 2009
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -229,6 +229,11 @@ public:
       virtual size_t size(void) const {
         return sizeof(Choice);
       }
+      /// Archive into \a e
+      virtual void archive(Archive& e) const {
+        Gecode::Choice::archive(e);
+        e.put(fail);
+      }
     };
     /// Construct brancher
     NestedSearch(Space& home) : Brancher(home), done(false) {
@@ -302,6 +307,10 @@ public:
 
       return new Choice(*this, fail);
     }
+    virtual Choice* choice(const Space&, Archive& e) {
+      bool fail; e >> fail;
+      return new Choice(*this, fail);
+    }
     virtual ExecStatus commit(Space&, const Gecode::Choice& _c, unsigned int) {
       return static_cast<const Choice&>(_c).fail ? ES_FAILED : ES_OK;
     }
