@@ -11,8 +11,8 @@
  *     Christian Schulte, 2007
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-06-16 16:13:18 +0200 (Thu, 16 Jun 2011) $ by $Author: schulte $
+ *     $Revision: 12053 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -248,8 +248,12 @@ public:
         rel(*this, h(p,w), IRT_LE, a(p,w));
 
     // Home teams in first week are ordered
-    for (int p=0; p<periods()-1; p++)
-      rel(*this, h(p,0), IRT_LE, h(p+1,0));
+    {
+      IntVarArgs h0(periods());
+      for (int p=0; p<periods(); p++)
+        h0[p] = h(p,0);
+      rel(*this, h0, IRT_LE);
+    }
 
     // Fix first pair
     rel(*this, h(0,0), IRT_EQ, 1);
