@@ -7,8 +7,8 @@
  *     Mikael Lagerkvist, 2008
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -336,6 +336,11 @@ public:
       virtual size_t size(void) const {
         return sizeof(Choice);
       }
+      /// Archive into \a e
+      virtual void archive(Archive& e) const {
+        Gecode::Choice::archive(e);
+        e << alternatives() << pos << val;
+      }
     };
 
     /// Construct brancher
@@ -388,6 +393,11 @@ public:
              val < sm.norders);
       return new Choice(*this, (val<firstzero) ? 2 : 1, pos, val);
     }
+    virtual Choice* choice(const Space&, Archive& e) {
+      unsigned int alt; int pos, val;
+      e >> alt >> pos >> val;
+      return new Choice(*this, alt, pos, val);
+    }
     /// Perform commit for choice \a _c and alternative \a a
     virtual ExecStatus commit(Space& home, const Gecode::Choice& _c,
                               unsigned int a) {
