@@ -7,8 +7,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2010-10-07 11:52:01 +0200 (Thu, 07 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11473 $
+ *     $Date: 2011-02-22 10:26:48 +0100 (Tue, 22 Feb 2011) $ by $Author: tack $
+ *     $Revision: 11761 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -101,8 +101,8 @@ public:
            * Using matching constraints
            */
 
-          channel(*this, IntVarArgs()<<x1<<x2<<x3, x);
-          channel(*this, IntVarArgs()<<y1<<y2<<y3, y);
+          channelSorted(*this, IntVarArgs()<<x1<<x2<<x3, x);
+          channelSorted(*this, IntVarArgs()<<y1<<y2<<y3, y);
         } else if (opt.model() == MODEL_SEQ) {
           SetVar sx1 = expr(*this, singleton(x1));
           SetVar sx2 = expr(*this, singleton(x2));
