@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-14 20:32:01 +0200 (Wed, 14 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11195 $
+ *     $Date: 2010-10-06 22:44:30 +0200 (Wed, 06 Oct 2010) $ by $Author: schulte $
+ *     $Revision: 11465 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -400,6 +400,36 @@ namespace Gecode {
     size(static_cast<unsigned int>(atoi(argv[1])));
   }
 
+
+
+  InstanceOptions::InstanceOptions(const char* e)
+    : Options(e), _inst(NULL) {}
+  
+  void
+  InstanceOptions::instance(const char* s) {
+    Driver::BaseOption::strdel(_inst);
+    _inst = strdup(s);
+  }
+
+  void
+  InstanceOptions::help(void) {
+    Options::help();
+    std::cerr << "\t(string) default: " << instance() << std::endl
+              << "\t\twhich instance for script" << std::endl;
+  }
+
+  void
+  InstanceOptions::parse(int& argc, char* argv[]) {
+    Options::parse(argc,argv);
+    if (argc < 2)
+      return;
+    instance(argv[1]);
+  }
+
+  InstanceOptions::~InstanceOptions(void) {
+    Driver::BaseOption::strdel(_inst);
+  }
+
 }
 
 // STATISTICS: driver-any
