@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-10-06 22:44:30 +0200 (Wed, 06 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11465 $
+ *     $Date: 2011-02-22 13:52:12 +0100 (Tue, 22 Feb 2011) $ by $Author: schulte $
+ *     $Revision: 11766 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -359,19 +359,19 @@ namespace Gecode {
 
   forceinline void
   Options::_I::click(Gist::Inspector* i) {
-    _click[n_click++] = i;
+    _click[static_cast<int>(n_click++)] = i;
   }
   forceinline void
   Options::_I::solution(Gist::Inspector* i) {
-    _solution[n_solution++] = i;
+    _solution[static_cast<int>(n_solution++)] = i;
   }
   forceinline void
   Options::_I::move(Gist::Inspector* i) {
-    _move[n_move++] = i;
+    _move[static_cast<int>(n_move++)] = i;
   }
   forceinline void
   Options::_I::compare(Gist::Comparator* i) {
-    _compare[n_compare++] = i;
+    _compare[static_cast<int>(n_compare++)] = i;
   }
   forceinline Gist::Inspector*
   Options::_I::click(unsigned int i) const {
