@@ -7,8 +7,8 @@
  *     Guido Tack, 2007
  *
  *  Last modified:
- *     $Date: 2010-10-09 13:58:12 +0200 (Sat, 09 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11498 $
+ *     $Date: 2011-09-21 11:18:38 +0200 (Wed, 21 Sep 2011) $ by $Author: tack $
+ *     $Revision: 12410 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -143,6 +143,16 @@ namespace Gecode { namespace FlatZinc {
         return INT_VAL_RND;
       if (s->id == "indomain")
         return INT_VALUES_MIN;
+      if (s->id == "indomain_middle") {
+        std::cerr << "Warning, replacing unsupported annotation "
+                  << "indomain_middle with indomain_median" << std::endl;
+        return INT_VAL_MED;
+      }
+      if (s->id == "indomain_interval") {
+        std::cerr << "Warning, replacing unsupported annotation "
+                  << "indomain_interval with indomain_split" << std::endl;
+        return INT_VAL_SPLIT_MIN;
+      }
     }
     std::cerr << "Warning, ignored search annotation: ";
     ann->print(std::cerr);
@@ -346,8 +356,13 @@ namespace Gecode { namespace FlatZinc {
   }
 
   void
-  FlatZincSpace::createBranchers(AST::Node* ann, bool ignoreUnknown,
+  FlatZincSpace::createBranchers(AST::Node* ann, int seed,
+                                 bool ignoreUnknown,
                                  std::ostream& err) {
+    VarBranchOptions varbo;
+    varbo.seed = seed;
+    ValBranchOptions valbo;
+    valbo.seed = seed;
     if (ann) {
       std::vector<AST::Node*> flatAnn;
       if (ann->isArray()) {
@@ -375,7 +390,8 @@ namespace Gecode { namespace FlatZinc {
               continue;
             va[k++] = iv[vars->a[i]->getIntVar()];
           }
-          branch(*this, va, ann2ivarsel(args->a[1]), ann2ivalsel(args->a[2]));
+          branch(*this, va, ann2ivarsel(args->a[1]), ann2ivalsel(args->a[2]),
+                 varbo,valbo);
         } catch (AST::TypeError& e) {
         try {
           AST::Call *call = flatAnn[i]->getCall("int_assign");
@@ -411,7 +427,7 @@ namespace Gecode { namespace FlatZinc {
               va[k++] = bv[vars->a[i]->getBoolVar()];
             }
             branch(*this, va, ann2ivarsel(args->a[1]), 
-                   ann2ivalsel(args->a[2]));        
+                   ann2ivalsel(args->a[2]), varbo, valbo);
           } catch (AST::TypeError& e) {
             (void) e;
 #ifdef GECODE_HAS_SET_VARS
@@ -431,7 +447,7 @@ namespace Gecode { namespace FlatZinc {
                 va[k++] = sv[vars->a[i]->getSetVar()];
               }
               branch(*this, va, ann2svarsel(args->a[1]), 
-                               ann2svalsel(args->a[2]));        
+                     ann2svalsel(args->a[2]), varbo, valbo);
             } catch (AST::TypeError& e) {
               (void) e;
               if (!ignoreUnknown) {
@@ -477,7 +493,7 @@ namespace Gecode { namespace FlatZinc {
         bv_sol[k++] = bv[i];
 
     branch(*this, iv_sol, INT_VAR_SIZE_AFC_MIN, INT_VAL_MIN);
-    branch(*this, bv_sol, INT_VAR_AFC_MIN, INT_VAL_MIN);
+    branch(*this, bv_sol, INT_VAR_AFC_MAX, INT_VAL_MIN);
 #ifdef GECODE_HAS_SET_VARS
     introduced = 0;
     for (int i=sv.size(); i--;)
@@ -493,7 +509,7 @@ namespace Gecode { namespace FlatZinc {
     branch(*this, sv_sol, SET_VAR_SIZE_AFC_MIN, SET_VAL_MIN_INC);
 #endif
     branch(*this, iv_tmp, INT_VAR_SIZE_AFC_MIN, INT_VAL_MIN);
-    branch(*this, bv_tmp, INT_VAR_AFC_MIN, INT_VAL_MIN);
+    branch(*this, bv_tmp, INT_VAR_AFC_MAX, INT_VAL_MIN);
 #ifdef GECODE_HAS_SET_VARS
     branch(*this, sv_tmp, SET_VAR_SIZE_AFC_MIN, SET_VAL_MIN_INC);
 #endif
