@@ -11,8 +11,8 @@
  *     Mikael Lagerkvist, 2009
  *
  *  Last modified:
- *     $Date: 2011-01-17 12:20:14 +0100 (Mon, 17 Jan 2011) $ by $Author: tack $
- *     $Revision: 11537 $
+ *     $Date: 2011-09-21 11:18:38 +0200 (Wed, 21 Sep 2011) $ by $Author: tack $
+ *     $Revision: 12410 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -42,8 +42,6 @@
 #include <gecode/flatzinc/registry.hh>
 #include <gecode/kernel.hh>
 #include <gecode/int.hh>
-#include <gecode/scheduling.hh>
-#include <gecode/graph.hh>
 #include <gecode/minimodel.hh>
 #ifdef GECODE_HAS_SET_VARS
 #include <gecode/set.hh>
@@ -228,15 +226,14 @@ namespace Gecode { namespace FlatZinc {
     }
 
     inline SetVarArgs arg2setvarargs(FlatZincSpace& s, AST::Node* arg,
-                                     int offset = 0) {
+                                     int offset = 0, int doffset = 0,
+                                     const IntSet& od=IntSet::empty) {
       AST::Array* a = arg->getArray();
-      if (a->a.size() == 0) {
-        SetVarArgs emptyIa(0);
-        return emptyIa;
-      }
       SetVarArgs ia(a->a.size()+offset);
-      for (int i=offset; i--;)
-        ia[i] = SetVar(s, IntSet::empty, IntSet::empty);
+      for (int i=offset; i--;) {
+        IntSet d = i<doffset ? od : IntSet::empty;
+        ia[i] = SetVar(s, d, d);
+      }
       for (int i=a->a.size(); i--;) {
         ia[i+offset] = getSetVar(s, a->a[i]);
       }
@@ -288,7 +285,8 @@ namespace Gecode { namespace FlatZinc {
 
     void p_distinct(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
       IntVarArgs va = arg2intvarargs(s, ce[0]);
-      distinct(s, va, ann2icl(ann));    
+      IntConLevel icl = ann2icl(ann);
+      distinct(s, va, icl == ICL_DEF ? ICL_BND : icl);
     }
     void p_distinctOffset(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
       IntVarArgs va = arg2intvarargs(s, ce[1]);
@@ -297,7 +295,8 @@ namespace Gecode { namespace FlatZinc {
       for (int i=offs->a.size(); i--; ) {
         oa[i] = offs->a[i]->getInt();    
       }
-      distinct(s, oa, va, ann2icl(ann));
+      IntConLevel icl = ann2icl(ann);
+      distinct(s, oa, va, icl == ICL_DEF ? ICL_BND : icl);
     }
 
     void p_all_equal(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
@@ -397,7 +396,8 @@ namespace Gecode { namespace FlatZinc {
               if (i != singleIntVar)
                 ia_tmp[count++] = ia[singleIntVar] == -1 ? ia[i] : -ia[i];
             }
-            linear(s, ia_tmp, iv, irt, siv, ann2icl(ann));
+            IntRelType t = (ia[singleIntVar] == -1 ? irt : swap(irt));
+            linear(s, ia_tmp, iv, t, siv, ann2icl(ann));
           } else {
             IntVarArgs iv = arg2intvarargs(s, ce[1]);
             linear(s, ia, iv, irt, ce[2]->getInt(), ann2icl(ann));
@@ -434,7 +434,8 @@ namespace Gecode { namespace FlatZinc {
               if (i != singleIntVar)
                 ia_tmp[count++] = ia[singleIntVar] == -1 ? ia[i] : -ia[i];
             }
-            linear(s, ia_tmp, iv, irt, siv, getBoolVar(s, ce[3]), 
+            IntRelType t = (ia[singleIntVar] == -1 ? irt : swap(irt));
+            linear(s, ia_tmp, iv, t, siv, getBoolVar(s, ce[3]), 
                    ann2icl(ann));
           } else {
             IntVarArgs iv = arg2intvarargs(s, ce[1]);
@@ -691,7 +692,9 @@ namespace Gecode { namespace FlatZinc {
 
 #define BOOL_ARRAY_OP(op) \
     BoolVarArgs bv = arg2boolvarargs(s, ce[0]); \
-    if (ce[1]->isBool()) { \
+    if (ce.size()==1) { \
+      rel(s, op, bv, 1, ann2icl(ann)); \
+    } else if (ce[1]->isBool()) { \
       rel(s, op, bv, ce[1]->getBool(), ann2icl(ann)); \
     } else { \
       rel(s, op, bv, s.bv[ce[1]->getBoolVar()], ann2icl(ann)); \
@@ -711,6 +714,10 @@ namespace Gecode { namespace FlatZinc {
     {
       BOOL_ARRAY_OP(BOT_OR);
     }
+    void p_array_bool_xor(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann)
+    {
+      BOOL_ARRAY_OP(BOT_XOR);
+    }
     void p_array_bool_clause(FlatZincSpace& s, const ConExpr& ce,
                              AST::Node* ann) {
       BoolVarArgs bvp = arg2boolvarargs(s, ce[0]);
@@ -814,6 +821,27 @@ namespace Gecode { namespace FlatZinc {
         dom(s, getIntVar(s, ce[0]), d);
       }
     }
+    void p_int_in_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
+      IntSet d = arg2intset(s,ce[1]);
+      if (ce[0]->isBoolVar()) {
+        IntSetRanges dr(d);
+        Iter::Ranges::Singleton sr(0,1);
+        Iter::Ranges::Inter<IntSetRanges,Iter::Ranges::Singleton> i(dr,sr);
+        IntSet d01(i);
+        if (d01.size() == 0) {
+          rel(s, getBoolVar(s, ce[2]) == 0);
+        } else if (d01.max() == 0) {
+          rel(s, getBoolVar(s, ce[2]) == !getBoolVar(s, ce[0]));
+        } else if (d01.min() == 1) {
+          rel(s, getBoolVar(s, ce[2]) == getBoolVar(s, ce[0]));
+        } else {
+          rel(s, getBoolVar(s, ce[2]) == 1);
+        }
+      } else {
+        std::cerr << "in_in_reif("<<getIntVar(s, ce[0])<<","<<d<<","<<getBoolVar(s, ce[2])<<")\n";
+        dom(s, getIntVar(s, ce[0]), d, getBoolVar(s, ce[2]));
+      }
+    }
 
     /* constraints from the standard library */
   
@@ -902,11 +930,11 @@ namespace Gecode { namespace FlatZinc {
       IntVarRanges* iv0_ri = re.alloc<IntVarRanges>(iv0.size());
       for (int i=iv0.size(); i--;)
         iv0_ri[i] = IntVarRanges(iv0[i]);
-      Iter::Ranges::NaryUnion<IntVarRanges> iv0_r(re,iv0_ri,iv0.size());
-      Iter::Ranges::Diff<Iter::Ranges::NaryUnion<IntVarRanges>,IntSetRanges> 
+      Iter::Ranges::NaryUnion iv0_r(re,iv0_ri,iv0.size());
+      Iter::Ranges::Diff<Iter::Ranges::NaryUnion,IntSetRanges> 
         extra_r(iv0_r,cover_r);
       Iter::Ranges::ToValues<Iter::Ranges::Diff<
-        Iter::Ranges::NaryUnion<IntVarRanges>,IntSetRanges> > extra(extra_r);
+        Iter::Ranges::NaryUnion,IntSetRanges> > extra(extra_r);
       for (; extra(); ++extra) {
         cover << extra.val();
         iv1 << IntVar(s,0,iv0.size());
@@ -938,8 +966,8 @@ namespace Gecode { namespace FlatZinc {
       IntVarRanges* xrs = re.alloc<IntVarRanges>(x.size());
       for (int i=x.size(); i--;)
         xrs[i].init(x[i]);
-      Iter::Ranges::NaryUnion<IntVarRanges> u(re, xrs, x.size());
-      Iter::Ranges::ToValues<Iter::Ranges::NaryUnion<IntVarRanges> > uv(u);
+      Iter::Ranges::NaryUnion u(re, xrs, x.size());
+      Iter::Ranges::ToValues<Iter::Ranges::NaryUnion> uv(u);
       for (; uv(); ++uv) {
         if (!cover_s.in(uv.val())) {
           cover << uv.val();
@@ -1132,47 +1160,38 @@ namespace Gecode { namespace FlatZinc {
         // Unary
         if (duration.assigned()) {
           IntArgs durationI(n);
-          bool allone = true;
-          for (int i=n; i--;) {
-            durationI[i] = duration[i].val();
-            if (durationI[i] != 1)
-              allone = false;
-          }
-          if (allone) {
-            distinct(s,start,ann2icl(ann));
-          } else {
-            unary(s,start,durationI);
-          }
-        } else {
-          IntVarArgs end(s,n,Int::Limits::min,Int::Limits::max);
           for (int i=n; i--;)
-            rel(s,start[i]+duration[i]==end[i]);
+            durationI[i] = duration[i].val();
+          unary(s,start,durationI);
+        } else {
+          IntVarArgs end(n);
+          for (int i=n; i--;)
+            end[i] = expr(s,start[i]+duration[i]);
           unary(s,start,duration,end);
         }
-      } else if (bound.assigned()) {
-        if (height.assigned()) {
-          IntArgs heightI(n);
+      } else if (height.assigned()) {
+        IntArgs heightI(n);
+        for (int i=n; i--;)
+          heightI[i] = height[i].val();
+        if (duration.assigned()) {
+          IntArgs durationI(n);
           for (int i=n; i--;)
-            heightI[i] = height[i].val();
-          if (duration.assigned()) {
-            IntArgs durationI(n);
-            for (int i=n; i--;)
-              durationI[i] = duration[i].val();
-            cumulative(s, bound.val(), start, durationI, heightI);
-          } else {
-            IntVarArgs end(n);
-            for (int i = n; i--; )
-              end[i] = expr(s, start[i]+duration[i]);
-            cumulative(s, bound.val(), start, duration, end, heightI);
-          }
+            durationI[i] = duration[i].val();
+          cumulative(s, bound, start, durationI, heightI);
         } else {
-          IntArgs machine = IntArgs::create(n,0,0);
-          IntArgs limit(1, bound.val());
           IntVarArgs end(n);
-          for (int i = n; i--; ) end[i] = IntVar(s, 0, Int::Limits::max);
-          cumulatives(s, machine, start, duration, end, height, limit, true,
-                      ann2icl(ann));
-        }        
+          for (int i = n; i--; )
+            end[i] = expr(s,start[i]+duration[i]);
+          cumulative(s, bound, start, duration, end, heightI);
+        }
+      } else if (bound.assigned()) {
+        IntArgs machine = IntArgs::create(n,0,0);
+        IntArgs limit(1, bound.val());
+        IntVarArgs end(n);
+        for (int i=n; i--;)
+          end[i] = expr(s,start[i]+duration[i]);
+        cumulatives(s, machine, start, duration, end, height, limit, true,
+                    ann2icl(ann));
       } else {
         int min = Gecode::Int::Limits::max;
         int max = Gecode::Int::Limits::min;
@@ -1248,6 +1267,72 @@ namespace Gecode { namespace FlatZinc {
       circuit(s,c,xv,z,ann2icl(ann));
     }
 
+    void p_nooverlap(FlatZincSpace& s, const ConExpr& ce, AST::Node *ann) {
+      IntVarArgs x0 = arg2intvarargs(s, ce[0]);
+      IntVarArgs w = arg2intvarargs(s, ce[1]);
+      IntVarArgs y0 = arg2intvarargs(s, ce[2]);
+      IntVarArgs h = arg2intvarargs(s, ce[3]);
+      IntVarArgs x1(x0.size()), y1(y0.size());
+      for (int i=x0.size(); i--; )
+        x1[i] = expr(s, x0[i] + w[i]);
+      for (int i=y0.size(); i--; )
+        y1[i] = expr(s, y0[i] + h[i]);
+      nooverlap(s,x0,w,x1,y0,h,y1,ann2icl(ann));
+    }
+
+    void p_precede(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
+      IntVarArgs x = arg2intvarargs(s, ce[0]);
+      IntArgs c = arg2intargs(ce[1]);
+      precede(s,x,c,ann2icl(ann));
+    }
+
+    void p_nvalue(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
+      IntVarArgs x = arg2intvarargs(s, ce[1]);
+      if (ce[0]->isIntVar()) {
+        IntVar y = getIntVar(s,ce[0]);
+        nvalues(s,x,IRT_EQ,y,ann2icl(ann));
+      } else {
+        nvalues(s,x,IRT_EQ,ce[0]->getInt(),ann2icl(ann));
+      }
+    }
+
+    void p_among(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
+      IntVarArgs x = arg2intvarargs(s, ce[1]);
+      IntSet v = arg2intset(s, ce[2]);
+      if (ce[0]->isIntVar()) {
+        IntVar n = getIntVar(s,ce[0]);
+        std::cerr << "count " << n << std::endl;
+        count(s,x,v,IRT_EQ,n,ann2icl(ann));
+      } else {
+        std::cerr << "count i " << x << " " << v << " " << ce[0]->getInt() << std::endl;
+        count(s,x,v,IRT_EQ,ce[0]->getInt(),ann2icl(ann));
+      }
+    }
+
+    void p_member_int(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
+      IntVarArgs x = arg2intvarargs(s, ce[0]);
+      IntVar y = getIntVar(s, ce[1]);
+      member(s,x,y,ann2icl(ann));
+    }
+    void p_member_int_reif(FlatZincSpace& s, const ConExpr& ce,
+                           AST::Node* ann) {
+      IntVarArgs x = arg2intvarargs(s, ce[0]);
+      IntVar y = getIntVar(s, ce[1]);
+      BoolVar b = getBoolVar(s, ce[2]);
+      member(s,x,y,b,ann2icl(ann));
+    }
+    void p_member_bool(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
+      BoolVarArgs x = arg2boolvarargs(s, ce[0]);
+      BoolVar y = getBoolVar(s, ce[1]);
+      member(s,x,y,ann2icl(ann));
+    }
+    void p_member_bool_reif(FlatZincSpace& s, const ConExpr& ce,
+                            AST::Node* ann) {
+      BoolVarArgs x = arg2boolvarargs(s, ce[0]);
+      BoolVar y = getBoolVar(s, ce[1]);
+      member(s,x,y,getBoolVar(s, ce[2]),ann2icl(ann));
+    }
+
     class IntPoster {
     public:
       IntPoster(void) {
@@ -1304,6 +1389,7 @@ namespace Gecode { namespace FlatZinc {
         registry().add("bool_xor", &p_bool_xor);
         registry().add("array_bool_and", &p_array_bool_and);
         registry().add("array_bool_or", &p_array_bool_or);
+        registry().add("array_bool_xor", &p_array_bool_xor);
         registry().add("bool_clause", &p_array_bool_clause);
         registry().add("bool_clause_reif", &p_array_bool_clause_reif);
         registry().add("bool_left_imp", &p_bool_l_imp);
@@ -1315,6 +1401,11 @@ namespace Gecode { namespace FlatZinc {
         registry().add("array_var_bool_element", &p_array_bool_element);
         registry().add("bool2int", &p_bool2int);
         registry().add("int_in", &p_int_in);
+        registry().add("int_in_reif", &p_int_in_reif);
+#ifndef GECODE_HAS_SET_VARS
+        registry().add("set_in", &p_int_in);
+        registry().add("set_in_reif", &p_int_in_reif);
+#endif
       
         registry().add("array_int_lt", &p_array_int_lt);
         registry().add("array_int_lq", &p_array_int_lq);
@@ -1322,8 +1413,8 @@ namespace Gecode { namespace FlatZinc {
         registry().add("at_least_int", &p_at_least);
         registry().add("at_most_int", &p_at_most);
         registry().add("bin_packing_load_gecode", &p_bin_packing_load);
-        registry().add("global_cardinality_gecode", &p_global_cardinality);
-        registry().add("global_cardinality_closed_gecode",
+        registry().add("global_cardinality", &p_global_cardinality);
+        registry().add("global_cardinality_closed",
           &p_global_cardinality_closed);
         registry().add("global_cardinality_low_up", 
           &p_global_cardinality_low_up);
@@ -1361,9 +1452,17 @@ namespace Gecode { namespace FlatZinc {
         registry().add("schedule_unary", &p_schedule_unary);
         registry().add("schedule_unary_optional", &p_schedule_unary_optional);
 
-        registry().add("circuit", &p_circuit);
-        registry().add("circuit_cost_array", &p_circuit_cost_array);
-        registry().add("circuit_cost", &p_circuit_cost);
+        registry().add("gecode_circuit", &p_circuit);
+        registry().add("gecode_circuit_cost_array", &p_circuit_cost_array);
+        registry().add("gecode_circuit_cost", &p_circuit_cost);
+        registry().add("gecode_nooverlap", &p_nooverlap);
+        registry().add("gecode_precede", &p_precede);
+        registry().add("nvalue",&p_nvalue);
+        registry().add("among",&p_among);
+        registry().add("member_int",&p_member_int);
+        registry().add("member_int_reif_gecode",&p_member_int_reif);
+        registry().add("member_bool",&p_member_bool);
+        registry().add("member_bool_reif_gecode",&p_member_bool_reif);
       }
     };
     IntPoster __int_poster;
@@ -1414,17 +1513,27 @@ namespace Gecode { namespace FlatZinc {
     }
 
 
+    void p_set_rel(FlatZincSpace& s, SetRelType srt, const ConExpr& ce) {
+      rel(s, getSetVar(s, ce[0]), srt, getSetVar(s, ce[1]));
+    }
+
     void p_set_eq(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_EQ, getSetVar(s, ce[1]));
+      p_set_rel(s, SRT_EQ, ce);
     }
     void p_set_ne(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_NQ, getSetVar(s, ce[1]));
+      p_set_rel(s, SRT_NQ, ce);
     }
     void p_set_subset(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_SUB, getSetVar(s, ce[1]));
+      p_set_rel(s, SRT_SUB, ce);
     }
     void p_set_superset(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_SUP, getSetVar(s, ce[1]));
+      p_set_rel(s, SRT_SUP, ce);
+    }
+    void p_set_le(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
+      p_set_rel(s, SRT_LQ, ce);
+    }
+    void p_set_lt(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
+      p_set_rel(s, SRT_LE, ce);
     }
     void p_set_card(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
       if (!ce[1]->isIntVar()) {
@@ -1459,23 +1568,30 @@ namespace Gecode { namespace FlatZinc {
         }
       }
     }
-    void p_set_eq_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_EQ, getSetVar(s, ce[1]),
+    void p_set_rel_reif(FlatZincSpace& s, SetRelType srt, const ConExpr& ce) {
+      rel(s, getSetVar(s, ce[0]), srt, getSetVar(s, ce[1]),
           getBoolVar(s, ce[2]));
     }
+
+    void p_set_eq_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
+      p_set_rel_reif(s,SRT_EQ,ce);
+    }
+    void p_set_le_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
+      p_set_rel_reif(s,SRT_LQ,ce);
+    }
+    void p_set_lt_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
+      p_set_rel_reif(s,SRT_LE,ce);
+    }
     void p_set_ne_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_NQ, getSetVar(s, ce[1]),
-          getBoolVar(s, ce[2]));
+      p_set_rel_reif(s,SRT_NQ,ce);
     }
     void p_set_subset_reif(FlatZincSpace& s, const ConExpr& ce,
                            AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_SUB, getSetVar(s, ce[1]),
-          getBoolVar(s, ce[2]));
+      p_set_rel_reif(s,SRT_SUB,ce);
     }
     void p_set_superset_reif(FlatZincSpace& s, const ConExpr& ce,
                              AST::Node *) {
-      rel(s, getSetVar(s, ce[0]), SRT_SUP, getSetVar(s, ce[1]),
-          getBoolVar(s, ce[2]));
+      p_set_rel_reif(s,SRT_SUP,ce);
     }
     void p_set_in_reif(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
       if (!ce[1]->isSetVar()) {
@@ -1510,6 +1626,16 @@ namespace Gecode { namespace FlatZinc {
     void p_set_disjoint(FlatZincSpace& s, const ConExpr& ce, AST::Node *) {
       rel(s, getSetVar(s, ce[0]), SRT_DISJ, getSetVar(s, ce[1]));
     }
+    
+    void p_link_set_to_booleans(FlatZincSpace& s, const ConExpr& ce,
+                                AST::Node *) {
+      SetVar x = getSetVar(s, ce[0]);
+      int idx = ce[2]->getInt();
+      assert(idx >= 0);
+      rel(s, x || IntSet(Set::Limits::min,idx-1));
+      BoolVarArgs y = arg2boolvarargs(s,ce[1],idx);
+      channel(s, y, x);
+    }
 
     void p_array_set_element(FlatZincSpace& s, const ConExpr& ce,
                              AST::Node*) {
@@ -1592,10 +1718,46 @@ namespace Gecode { namespace FlatZinc {
       sequence(s, sv, getSetVar(s, ce[1]));
     }
 
+    void p_int_set_channel(FlatZincSpace& s, const ConExpr& ce,
+                           AST::Node *) {
+      int xoff=ce[1]->getInt();
+      assert(xoff >= 0);
+      int yoff=ce[3]->getInt();
+      assert(yoff >= 0);
+      IntVarArgs xv = arg2intvarargs(s, ce[0], xoff);
+      SetVarArgs yv = arg2setvarargs(s, ce[2], yoff, 1, IntSet(0, xoff-1));
+      IntSet xd(yoff,yv.size()-1);
+      for (int i=xoff; i<xv.size(); i++) {
+        dom(s, xv[i], xd);
+      }
+      IntSet yd(xoff,xv.size()-1);
+      for (int i=yoff; i<yv.size(); i++) {
+        dom(s, yv[i], SRT_SUB, yd);
+      }
+      channel(s,xv,yv);
+    }
+    
+    void p_range(FlatZincSpace& s, const ConExpr& ce, AST::Node*) {
+      int xoff=ce[1]->getInt();
+      assert(xoff >= 0);
+      IntVarArgs xv = arg2intvarargs(s,ce[0],xoff);
+      element(s, SOT_UNION, xv, getSetVar(s,ce[2]), getSetVar(s,ce[3]));
+    }
+    
+    void p_weights(FlatZincSpace& s, const ConExpr& ce, AST::Node*) {
+      IntArgs e = arg2intargs(ce[0]);
+      IntArgs w = arg2intargs(ce[1]);
+      SetVar x = getSetVar(s,ce[2]);
+      IntVar y = getIntVar(s,ce[3]);
+      weights(s,e,w,x,y);
+    }
+    
     class SetPoster {
     public:
       SetPoster(void) {
         registry().add("set_eq", &p_set_eq);
+        registry().add("set_le", &p_set_le);
+        registry().add("set_lt", &p_set_lt);
         registry().add("equal", &p_set_eq);
         registry().add("set_ne", &p_set_ne);
         registry().add("set_union", &p_set_union);
@@ -1609,12 +1771,16 @@ namespace Gecode { namespace FlatZinc {
         registry().add("set_card", &p_set_card);
         registry().add("set_in", &p_set_in);
         registry().add("set_eq_reif", &p_set_eq_reif);
+        registry().add("set_le_reif", &p_set_le_reif);
+        registry().add("set_lt_reif", &p_set_lt_reif);
         registry().add("equal_reif", &p_set_eq_reif);
         registry().add("set_ne_reif", &p_set_ne_reif);
         registry().add("set_subset_reif", &p_set_subset_reif);
         registry().add("set_superset_reif", &p_set_superset_reif);
         registry().add("set_in_reif", &p_set_in_reif);
         registry().add("disjoint", &p_set_disjoint);
+        registry().add("link_set_to_booleans_gecode", 
+                       &p_link_set_to_booleans);
 
         registry().add("array_set_union", &p_array_set_union);
         registry().add("array_set_partition", &p_array_set_partition);
@@ -1629,6 +1795,12 @@ namespace Gecode { namespace FlatZinc {
                        &p_array_set_element_intersect_in);
         registry().add("array_set_element_partition", 
                        &p_array_set_element_partition);
+        registry().add("int_set_channel_gecode", 
+                       &p_int_set_channel);
+        registry().add("range_gecode",
+                       &p_range);
+        registry().add("set_weights_gecode",
+                       &p_weights);
       }
     };
     SetPoster __set_poster;
