@@ -7,8 +7,8 @@
  *     Guido Tack, 2006
  *
  *  Last modified:
- *     $Date: 2010-08-12 13:38:27 +0200 (Thu, 12 Aug 2010) $ by $Author: zayenz $
- *     $Revision: 11349 $
+ *     $Date: 2011-05-02 15:44:28 +0200 (Mon, 02 May 2011) $ by $Author: tack $
+ *     $Revision: 11981 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -147,7 +147,7 @@ namespace Gecode { namespace Gist {
   forceinline
   Node::Node(int p, bool failed) : parent(p) {
     childrenOrFirstChild = NULL;
-    noOfChildren = NULL;
+    noOfChildren = 0;
     setTag(failed ? LEAF : UNDET);
   }
 
