@@ -7,8 +7,8 @@
  *     Guido Tack, 2006
  *
  *  Last modified:
- *     $Date: 2010-08-12 10:29:27 +0200 (Thu, 12 Aug 2010) $ by $Author: tack $
- *     $Revision: 11346 $
+ *     $Date: 2011-05-31 11:01:28 +0200 (Tue, 31 May 2011) $ by $Author: tack $
+ *     $Revision: 12032 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -453,9 +453,9 @@ namespace Gecode { namespace Gist {
     inspectBeforeFPGroup->addAction(ibfpa);
 
     if (inspectGroup->actions().size() < 10) {
-      ia->setShortcut(QKeySequence(
+      ia->setShortcut(QKeySequence(QString("Ctrl+")+
         QString("").setNum(inspectGroup->actions().size())));
-      ibfpa->setShortcut(QKeySequence(QString("Ctrl+")+
+      ibfpa->setShortcut(QKeySequence(QString("Ctrl+Alt+")+
         QString("").setNum(inspectBeforeFPGroup->actions().size())));
     }
   }
@@ -574,7 +574,6 @@ namespace Gecode { namespace Gist {
       toggleStop->setEnabled(false);
       unstopAll->setEnabled(false);
       
-      zoomToFit->setEnabled(false);
       center->setEnabled(false);
       exportPDF->setEnabled(false);
       exportWholeTreePDF->setEnabled(false);
@@ -656,7 +655,6 @@ namespace Gecode { namespace Gist {
       psv.run();
       navPrevSol->setEnabled(psv.getCursor().node() != root);
 
-      zoomToFit->setEnabled(true);
       center->setEnabled(true);
       exportPDF->setEnabled(true);
       exportWholeTreePDF->setEnabled(true);
