@@ -7,8 +7,8 @@
  *     Guido Tack, 2006
  *
  *  Last modified:
- *     $Date: 2010-09-30 09:44:05 +0200 (Thu, 30 Sep 2010) $ by $Author: tack $
- *     $Revision: 11431 $
+ *     $Date: 2011-08-30 11:40:00 +0200 (Tue, 30 Aug 2011) $ by $Author: tack $
+ *     $Revision: 12374 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -67,6 +67,8 @@ namespace Gecode { namespace Gist {
       QMutexLocker locker(&mutex);
       curBest = (bab ? new BestNode(NULL) : NULL);
       if (rootSpace->status() == SS_FAILED) {
+        if (!opt.clone)
+          delete rootSpace;
         rootSpace = NULL;
       } else {
         rootSpace = Gecode::Search::snapshot(rootSpace,opt);
@@ -91,6 +93,8 @@ namespace Gecode { namespace Gist {
       connect(&searcher, SIGNAL(solution(const Space*)),
               this, SIGNAL(solution(const Space*)),
               Qt::BlockingQueuedConnection);
+      connect(this, SIGNAL(solution(const Space*)),
+              this, SLOT(inspectSolution(const Space*)));
       connect(&searcher, SIGNAL(solution(const Space*)),
               this, SLOT(inspectSolution(const Space*)),
               Qt::BlockingQueuedConnection);
@@ -638,7 +642,6 @@ namespace Gecode { namespace Gist {
             if (currentNode->getStatus() == SOLVED) {
               assert(currentNode->hasCopy());
               emit solution(currentNode->getWorkingSpace());
-              currentNode->purge(*na);
             }
             emit statusChanged(currentNode,stats,true);
             for (int i=0; i<moveInspectors.size(); i++) {
@@ -650,6 +653,9 @@ namespace Gecode { namespace Gist {
                 failedInspectorType = -1;
               }
             }
+            if (currentNode->getStatus() == SOLVED) {
+              currentNode->purge(*na);
+            }
           }
           break;
       case FAILED:
@@ -766,16 +772,6 @@ namespace Gecode { namespace Gist {
           failedInspectorType = -1;
         }
       }
-      for (int i=0; i<moveInspectors.size(); i++) {
-        if (moveInspectors[i].second) {
-          if (c == NULL)
-            c = s->clone();
-          failedInspectorType = 0;
-          failedInspector = i;
-          moveInspectors[i].first->inspect(*c);
-          failedInspectorType = -1;
-        }
-      }
       delete c;
     } catch (Exception e) {
       switch (failedInspectorType) {
