@@ -9,8 +9,8 @@
  *     Guido Tack, 2006
  *
  *  Last modified:
- *     $Date: 2010-07-28 16:13:53 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11292 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -197,10 +197,9 @@ namespace Gecode { namespace Int { namespace Arithmetic {
       Positive<ViewRanges<View> > p(i);
       Negative<ViewRanges<View> > n(j);
 
-      Minus<Negative<ViewRanges<View> > > m(r,n);
+      Minus m(r,n);
 
-      Union<Positive<ViewRanges<View> >,
-        Minus<Negative<ViewRanges<View> > > > u(p,m);
+      Union<Positive<ViewRanges<View> >,Minus> u(p,m);
 
       GECODE_ME_CHECK(x1.inter_r(home,u,false));
 
@@ -210,9 +209,9 @@ namespace Gecode { namespace Int { namespace Arithmetic {
       ViewRanges<View> i(x1), j(x1);
 
       using namespace Iter::Ranges;
-      Minus<ViewRanges<View> > m(r,j);
+      Minus m(r,j);
 
-      Union<ViewRanges<View>,Minus<ViewRanges<View> > > u(i,m);
+      Union<ViewRanges<View>,Minus> u(i,m);
 
       GECODE_ME_CHECK(x0.inter_r(home,u,false));
     }
