@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-09-03 16:36:31 +0200 (Fri, 03 Sep 2010) $ by $Author: schulte $
- *     $Revision: 11390 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -379,7 +379,7 @@ namespace Gecode { namespace Int { namespace Arithmetic {
     for (int i = x.size(); i--; ) {
       ViewRanges<View> i_xi(x[i]); i_x[i]=i_xi;
     }
-    Iter::Ranges::NaryUnion<ViewRanges<View> > u(r, i_x, x.size());
+    Iter::Ranges::NaryUnion u(r, i_x, x.size());
     GECODE_ME_CHECK(y.inter_r(home,u,false));
     for (int i = x.size(); i--; )
       if (rtest_nq_dom(x[i],y) == RT_TRUE) {
