@@ -7,8 +7,8 @@
  *     Christian Schulte, 2010
  *
  *  Last modified:
- *     $Date: 2010-10-06 23:20:35 +0200 (Wed, 06 Oct 2010) $ by $Author: schulte $
- *     $Revision: 11468 $
+ *     $Date: 2011-06-07 22:37:43 +0200 (Tue, 07 Jun 2011) $ by $Author: schulte $
+ *     $Revision: 12049 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -207,6 +207,11 @@ namespace Gecode { namespace Int { namespace BinPacking {
 
     }
 
+    // Only if the propagator is at fixpoint here, continue with the more
+    // expensive stage for propagation.
+    if (IntView::me(modeventdelta()) != ME_INT_NONE)
+      return ES_NOFIX;
+
     // Now the invariant holds that no more assigned bins exist!
     {
       Region region(home);
