@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2009-10-20 13:28:57 +0200 (Tue, 20 Oct 2009) $ by $Author: schulte $
- *     $Revision: 9975 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -99,14 +99,18 @@ namespace Gecode { namespace Int { namespace Branch {
     return (a == 0) ? x.eq(home,n) : x.nq(home,n);
   }
   template<class View>
-  forceinline Support::RandomGenerator
+  forceinline typename ValRnd<View>::Choice
   ValRnd<View>::choice(Space&) {
     return r;
   }
   template<class View>
+  forceinline typename ValRnd<View>::Choice
+  ValRnd<View>::choice(const Space&, Archive& e) {
+    return Choice(e.get());
+  }
+  template<class View>
   forceinline void
-  ValRnd<View>::commit(Space&, const Support::RandomGenerator& c,
-                       unsigned int) {
+  ValRnd<View>::commit(Space&, const Choice& c, unsigned int) {
     r = c;
   }
   template<class View>
