@@ -7,8 +7,8 @@
  *     Christian Schulte, 2008
  *
  *  Last modified:
- *     $Date: 2010-09-01 12:02:08 +0200 (Wed, 01 Sep 2010) $ by $Author: schulte $
- *     $Revision: 11371 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -61,12 +61,18 @@ namespace Gecode { namespace Int { namespace Branch {
     /// Initialize choice for brancher \a b, position \a p, view choice \a viewc, and view \a x
     PosValuesChoice(const Brancher& b, const Pos& p,
                     const typename ViewSel::Choice& viewc, View x);
+    /// Initialize choice for brancher \a b from \a e
+    PosValuesChoice(const Brancher& b, unsigned int alt, Pos p,
+                    const typename ViewSel::Choice& viewc,
+                    Archive& e);
     /// Return value to branch with for alternative \a a
     int val(unsigned int a) const;
     /// Report size occupied
     virtual size_t size(void) const;
     /// Deallocate
     virtual ~PosValuesChoice(void);
+    /// Archive into \a e
+    virtual void archive(Archive& e) const;
   };
 
 
@@ -89,6 +95,20 @@ namespace Gecode { namespace Int { namespace Branch {
     pm[i].pos = w;
   }
 
+  template<class ViewSel, class View>
+  forceinline
+  PosValuesChoice<ViewSel,View>::
+  PosValuesChoice(const Brancher& b, unsigned int alt, Pos p,
+                  const typename ViewSel::Choice& viewc, Archive& e)
+    : PosChoice<ViewSel>(b,alt,p,viewc) {
+    e >> n;
+    pm = heap.alloc<PosMin>(n+1);
+    for (unsigned int i=0; i<n+1; i++) {
+      e >> pm[i].pos;
+      e >> pm[i].min;
+    }
+  }
+
   template<class ViewSel, class View>
   forceinline int
   PosValuesChoice<ViewSel,View>::val(unsigned int a) const {
@@ -119,6 +139,16 @@ namespace Gecode { namespace Int { namespace Branch {
     heap.free<PosMin>(pm,n+1);
   }
 
+  template<class ViewSel, class View>
+  forceinline void
+  PosValuesChoice<ViewSel,View>::archive(Archive& e) const {
+    PosChoice<ViewSel>::archive(e);
+    e << this->alternatives() << n;
+    for (unsigned int i=0; i<n+1; i++) {
+      e << pm[i].pos;
+      e << pm[i].min;
+    }
+  }
 
   template<class ViewSel, class View>
   forceinline
@@ -157,6 +187,16 @@ namespace Gecode { namespace Int { namespace Branch {
       (*this,p,viewsel.choice(home),v);
   }
 
+  template<class ViewSel, class View>
+  const Choice*
+  ViewValuesBrancher<ViewSel,View>::choice(const Space& home, Archive& e) {
+    int p;
+    unsigned int alt;
+    e >> p >> alt;
+    return new PosValuesChoice<ViewSel,View>
+      (*this,alt,p,viewsel.choice(home,e),e);
+  }
+
   template<class ViewSel, class View>
   ExecStatus
   ViewValuesBrancher<ViewSel,View>
