@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2010-06-03 20:34:20 +0200 (Thu, 03 Jun 2010) $ by $Author: schulte $
- *     $Revision: 11017 $
+ *     $Date: 2011-09-20 13:58:39 +0200 (Tue, 20 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12404 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -36,6 +36,7 @@
  */
 
 #include <gecode/int/count.hh>
+#include <gecode/int/rel.hh>
 
 namespace Gecode {
 
@@ -45,33 +46,36 @@ namespace Gecode {
     using namespace Int;
     Limits::check(n,"Int::count");
     Limits::check(m,"Int::count");
+
     if (home.failed()) return;
+
     ViewArray<IntView> xv(home,x);
     ConstIntView y(n);
+
     switch (r) {
     case IRT_EQ:
       GECODE_ES_FAIL((Count::EqInt<IntView,ConstIntView>
-                           ::post(home,xv,y,m)));
+                      ::post(home,xv,y,m)));
       break;
     case IRT_NQ:
-      GECODE_ES_FAIL((Count::NqInt<IntView,ConstIntView>
-                           ::post(home,xv,y,m)));
+      {
+        IntVar z(home,0,x.size());
+        GECODE_ME_FAIL(IntView(z).nq(home,m));
+        GECODE_ES_FAIL((Count::EqView<IntView,ConstIntView,IntView,true,false>
+                        ::post(home,xv,y,z,0)));
+      }
       break;
     case IRT_LE:
-      GECODE_ES_FAIL((Count::LqInt<IntView,ConstIntView>
-                           ::post(home,xv,y,m-1)));
-      break;
+      m--; // FALL THROUGH
     case IRT_LQ:
       GECODE_ES_FAIL((Count::LqInt<IntView,ConstIntView>
-                           ::post(home,xv,y,m)));
+                      ::post(home,xv,y,m)));
       break;
     case IRT_GR:
-      GECODE_ES_FAIL((Count::GqInt<IntView,ConstIntView>
-                           ::post(home,xv,y,m+1)));
-      break;
+      m++; // FALL THROUGH
     case IRT_GQ:
       GECODE_ES_FAIL((Count::GqInt<IntView,ConstIntView>
-                           ::post(home,xv,y,m)));
+                      ::post(home,xv,y,m)));
       break;
     default:
       throw UnknownRelation("Int::count");
@@ -80,35 +84,94 @@ namespace Gecode {
 
   void
   count(Home home, const IntVarArgs& x, IntVar y,
-        IntRelType r, int m, IntConLevel) {
+        IntRelType r, int m, IntConLevel icl) {
     using namespace Int;
     Limits::check(m,"Int::count");
     if (home.failed()) return;
     ViewArray<IntView> xv(home,x);
+
     switch (r) {
     case IRT_EQ:
-      GECODE_ES_FAIL((Count::EqInt<IntView,IntView>
-                           ::post(home,xv,y,m)));
+      {
+        ConstIntView z(m);
+        if ((icl == ICL_DOM) || (icl == ICL_DEF))
+          GECODE_ES_FAIL((Count::EqView<IntView,IntView,ConstIntView,true,true>
+                          ::post(home,xv,y,z,0)));
+        else
+          GECODE_ES_FAIL((Count::EqView<IntView,IntView,ConstIntView,true,false>
+                          ::post(home,xv,y,z,0)));
+      }
       break;
     case IRT_NQ:
-      GECODE_ES_FAIL((Count::NqInt<IntView,IntView>
-                           ::post(home,xv,y,m)));
+      {
+        IntVar z(home,0,x.size());
+        GECODE_ME_FAIL(IntView(z).nq(home,m));
+        GECODE_ES_FAIL((Count::EqView<IntView,IntView,IntView,true,false>
+                        ::post(home,xv,y,z,0)));
+      }
       break;
     case IRT_LE:
-      GECODE_ES_FAIL((Count::LqInt<IntView,IntView>
-                           ::post(home,xv,y,m-1)));
-      break;
+      m--; // FALL THROUGH
     case IRT_LQ:
       GECODE_ES_FAIL((Count::LqInt<IntView,IntView>
-                           ::post(home,xv,y,m)));
+                      ::post(home,xv,y,m)));
       break;
     case IRT_GR:
-      GECODE_ES_FAIL((Count::GqInt<IntView,IntView>
-                           ::post(home,xv,y,m+1)));
-      break;
+      m++; // FALL THROUGH
     case IRT_GQ:
-      GECODE_ES_FAIL((Count::GqInt<IntView,IntView>
-                           ::post(home,xv,y,m)));
+      {
+        ConstIntView z(m);
+        if ((icl == ICL_DOM) || (icl == ICL_DEF))
+          GECODE_ES_FAIL((Count::GqView<IntView,IntView,ConstIntView,true,true>
+                          ::post(home,xv,y,z,0)));
+        else
+          GECODE_ES_FAIL((Count::GqView<IntView,IntView,ConstIntView,true,false>
+                          ::post(home,xv,y,z,0)));
+      }
+      break;
+    default:
+      throw UnknownRelation("Int::count");
+    }
+  }
+
+  void
+  count(Home home, const IntVarArgs& x, const IntSet& y,
+        IntRelType r, int m, IntConLevel) {
+    using namespace Int;
+
+    if (y.size() == 1) {
+      count(home,x,y.min(),r,m);
+      return;
+    }
+      
+    Limits::check(y.min(),"Int::count");
+    Limits::check(y.max(),"Int::count");
+    Limits::check(m,"Int::count");
+
+    if (home.failed()) return;
+
+    ViewArray<IntView> xv(home,x);
+    switch (r) {
+    case IRT_EQ:
+      GECODE_ES_FAIL((Count::EqInt<IntView,IntSet>::post(home,xv,y,m)));
+      break;
+    case IRT_NQ:
+      {
+        IntVar z(home,0,x.size());
+        GECODE_ME_FAIL(IntView(z).nq(home,m));
+        GECODE_ES_FAIL((Count::EqView<IntView,IntSet,IntView,true,false>
+                        ::post(home,xv,y,z,0)));
+      }
+      break;
+    case IRT_LE:
+      m--; // FALL THROUGH
+    case IRT_LQ:
+      GECODE_ES_FAIL((Count::LqInt<IntView,IntSet>::post(home,xv,y,m)));
+      break;
+    case IRT_GR:
+      m++; // FALL THROUGH
+    case IRT_GQ:
+      GECODE_ES_FAIL((Count::GqInt<IntView,IntSet>::post(home,xv,y,m)));
       break;
     default:
       throw UnknownRelation("Int::count");
@@ -128,31 +191,31 @@ namespace Gecode {
     for (int i=x.size(); i--; )
       xy[i] = OffsetView(x[i],-y[i]);
 
-    ZeroIntView z;
+    ZeroIntView zero;
     switch (r) {
     case IRT_EQ:
       GECODE_ES_FAIL((Count::EqInt<OffsetView,ZeroIntView>
-                           ::post(home,xy,z,m)));
+                      ::post(home,xy,zero,m)));
       break;
     case IRT_NQ:
-      GECODE_ES_FAIL((Count::NqInt<OffsetView,ZeroIntView>
-                           ::post(home,xy,z,m)));
+      {
+        IntVar z(home,0,x.size());
+        GECODE_ME_FAIL(IntView(z).nq(home,m));
+        GECODE_ES_FAIL((Count::EqView<OffsetView,ZeroIntView,IntView,true,false>
+                        ::post(home,xy,zero,z,0)));
+      }
       break;
     case IRT_LE:
-      GECODE_ES_FAIL((Count::LqInt<OffsetView,ZeroIntView>
-                           ::post(home,xy,z,m-1)));
-      break;
+      m--; // FALL THROUGH
     case IRT_LQ:
       GECODE_ES_FAIL((Count::LqInt<OffsetView,ZeroIntView>
-                           ::post(home,xy,z,m)));
+                      ::post(home,xy,zero,m)));
       break;
     case IRT_GR:
-      GECODE_ES_FAIL((Count::GqInt<OffsetView,ZeroIntView>
-                           ::post(home,xy,z,m+1)));
-      break;
+      m++; // FALL THROUGH
     case IRT_GQ:
       GECODE_ES_FAIL((Count::GqInt<OffsetView,ZeroIntView>
-                           ::post(home,xy,z,m)));
+                      ::post(home,xy,zero,m)));
       break;
     default:
       throw UnknownRelation("Int::count");
@@ -169,12 +232,16 @@ namespace Gecode {
     ConstIntView yv(n);
     switch (r) {
     case IRT_EQ:
-      GECODE_ES_FAIL((Count::EqView<IntView,ConstIntView,IntView,true>
-                           ::post(home,xv,yv,z,0)));
+      GECODE_ES_FAIL((Count::EqView<IntView,ConstIntView,IntView,true,false>
+                      ::post(home,xv,yv,z,0)));
       break;
     case IRT_NQ:
-      GECODE_ES_FAIL((Count::NqView<IntView,ConstIntView,IntView,true>
-                           ::post(home,xv,yv,z,0)));
+      {
+        IntVar nz(home,0,x.size());
+        GECODE_ES_FAIL(Rel::Nq<IntView>::post(home,z,nz));
+        GECODE_ES_FAIL((Count::EqView<IntView,ConstIntView,IntView,true,false>
+                        ::post(home,xv,yv,nz,0)));
+      }
       break;
     case IRT_LE:
       GECODE_ES_FAIL((Count::LqView<IntView,ConstIntView,IntView,true>
@@ -185,12 +252,12 @@ namespace Gecode {
                            ::post(home,xv,yv,z,0)));
       break;
     case IRT_GR:
-      GECODE_ES_FAIL((Count::GqView<IntView,ConstIntView,IntView,true>
-                           ::post(home,xv,yv,z,1)));
+      GECODE_ES_FAIL((Count::GqView<IntView,ConstIntView,IntView,true,false>
+                      ::post(home,xv,yv,z,1)));
       break;
     case IRT_GQ:
-      GECODE_ES_FAIL((Count::GqView<IntView,ConstIntView,IntView,true>
-                           ::post(home,xv,yv,z,0)));
+      GECODE_ES_FAIL((Count::GqView<IntView,ConstIntView,IntView,true,false>
+                      ::post(home,xv,yv,z,0)));
       break;
     default:
       throw UnknownRelation("Int::count");
@@ -199,34 +266,99 @@ namespace Gecode {
 
   void
   count(Home home, const IntVarArgs& x, IntVar y,
-        IntRelType r, IntVar z, IntConLevel) {
+        IntRelType r, IntVar z, IntConLevel icl) {
     using namespace Int;
     if (home.failed()) return;
     ViewArray<IntView> xv(home,x);
     switch (r) {
     case IRT_EQ:
-      GECODE_ES_FAIL((Count::EqView<IntView,IntView,IntView,true>
-                           ::post(home,xv,y,z,0)));
+      if ((icl == ICL_DOM) || (icl == ICL_DEF))
+        GECODE_ES_FAIL((Count::EqView<IntView,IntView,IntView,true,true>
+                        ::post(home,xv,y,z,0)));
+      else
+        GECODE_ES_FAIL((Count::EqView<IntView,IntView,IntView,true,false>
+                        ::post(home,xv,y,z,0)));
       break;
     case IRT_NQ:
-      GECODE_ES_FAIL((Count::NqView<IntView,IntView,IntView,true>
-                           ::post(home,xv,y,z,0)));
+      {
+        IntVar nz(home,0,x.size());
+        GECODE_ES_FAIL(Rel::Nq<IntView>::post(home,z,nz));
+        GECODE_ES_FAIL((Count::EqView<IntView,IntView,IntView,true,false>
+                        ::post(home,xv,y,nz,0)));
+      }
       break;
     case IRT_LE:
       GECODE_ES_FAIL((Count::LqView<IntView,IntView,IntView,true>
-                           ::post(home,xv,y,z,-1)));
+                      ::post(home,xv,y,z,-1)));
       break;
     case IRT_LQ:
       GECODE_ES_FAIL((Count::LqView<IntView,IntView,IntView,true>
-                           ::post(home,xv,y,z,0)));
+                      ::post(home,xv,y,z,0)));
       break;
     case IRT_GR:
-      GECODE_ES_FAIL((Count::GqView<IntView,IntView,IntView,true>
-                           ::post(home,xv,y,z,1)));
+      if ((icl == ICL_DOM) || (icl == ICL_DEF))
+        GECODE_ES_FAIL((Count::GqView<IntView,IntView,IntView,true,true>
+                        ::post(home,xv,y,z,1)));
+      else
+        GECODE_ES_FAIL((Count::GqView<IntView,IntView,IntView,true,false>
+                        ::post(home,xv,y,z,0)));
       break;
     case IRT_GQ:
-      GECODE_ES_FAIL((Count::GqView<IntView,IntView,IntView,true>
-                           ::post(home,xv,y,z,0)));
+      if ((icl == ICL_DOM) || (icl == ICL_DEF))
+        GECODE_ES_FAIL((Count::GqView<IntView,IntView,IntView,true,true>
+                        ::post(home,xv,y,z,0)));
+      else
+        GECODE_ES_FAIL((Count::GqView<IntView,IntView,IntView,true,false>
+                        ::post(home,xv,y,z,0)));
+      break;
+    default:
+      throw UnknownRelation("Int::count");
+    }
+  }
+
+  void
+  count(Home home, const IntVarArgs& x, const IntSet& y,
+        IntRelType r, IntVar z, IntConLevel) {
+    using namespace Int;
+
+    if (y.size() == 1) {
+      count(home,x,y.min(),r,z);
+      return;
+    }
+      
+    Limits::check(y.min(),"Int::count");
+    Limits::check(y.max(),"Int::count");
+
+    if (home.failed()) return;
+    ViewArray<IntView> xv(home,x);
+    switch (r) {
+    case IRT_EQ:
+      GECODE_ES_FAIL((Count::EqView<IntView,IntSet,IntView,true,false>
+                      ::post(home,xv,y,z,0)));
+      break;
+    case IRT_NQ:
+      {
+        IntVar nz(home,0,x.size());
+        GECODE_ES_FAIL(Rel::Nq<IntView>::post(home,z,nz));
+        GECODE_ES_FAIL((Count::EqView<IntView,IntSet,IntView,true,false>
+                        ::post(home,xv,y,nz,0)));
+      }
+      break;
+    case IRT_LE:
+      GECODE_ES_FAIL((Count::LqView<IntView,IntSet,IntView,true>
+                      ::post(home,xv,y,z,-1)));
+      break;
+    case IRT_LQ:
+      GECODE_ES_FAIL((Count::LqView<IntView,IntSet,IntView,true>
+                      ::post(home,xv,y,z,0)));
+      break;
+    case IRT_GR:
+      GECODE_ES_FAIL((Count::GqView<IntView,IntSet,IntView,true,false>
+                      ::post(home,xv,y,z,1)));
+      break;
+    case IRT_GQ:
+      GECODE_ES_FAIL((Count::GqView<IntView,IntSet,IntView,true,false>
+                      ::post(home,xv,y,z,0)));
       break;
     default:
       throw UnknownRelation("Int::count");
@@ -248,28 +380,32 @@ namespace Gecode {
     ZeroIntView u;
     switch (r) {
     case IRT_EQ:
-      GECODE_ES_FAIL((Count::EqView<OffsetView,ZeroIntView,IntView,true>
-                           ::post(home,xy,u,z,0)));
+      GECODE_ES_FAIL((Count::EqView<OffsetView,ZeroIntView,IntView,true,false>
+                      ::post(home,xy,u,z,0)));
       break;
     case IRT_NQ:
-      GECODE_ES_FAIL((Count::NqView<OffsetView,ZeroIntView,IntView,true>
-                           ::post(home,xy,u,z,0)));
+      {
+        IntVar nz(home,0,x.size());
+        GECODE_ES_FAIL(Rel::Nq<IntView>::post(home,z,nz));
+        GECODE_ES_FAIL((Count::EqView<OffsetView,ZeroIntView,IntView,true,false>
+                        ::post(home,xy,u,nz,0)));
+      }
       break;
     case IRT_LE:
       GECODE_ES_FAIL((Count::LqView<OffsetView,ZeroIntView,IntView,true>
-                           ::post(home,xy,u,z,-1)));
+                      ::post(home,xy,u,z,-1)));
       break;
     case IRT_LQ:
       GECODE_ES_FAIL((Count::LqView<OffsetView,ZeroIntView,IntView,true>
-                           ::post(home,xy,u,z,0)));
+                      ::post(home,xy,u,z,0)));
       break;
     case IRT_GR:
-      GECODE_ES_FAIL((Count::GqView<OffsetView,ZeroIntView,IntView,true>
-                           ::post(home,xy,u,z,1)));
+      GECODE_ES_FAIL((Count::GqView<OffsetView,ZeroIntView,IntView,true,false>
+                      ::post(home,xy,u,z,1)));
       break;
     case IRT_GQ:
-      GECODE_ES_FAIL((Count::GqView<OffsetView,ZeroIntView,IntView,true>
-                           ::post(home,xy,u,z,0)));
+      GECODE_ES_FAIL((Count::GqView<OffsetView,ZeroIntView,IntView,true,false>
+                      ::post(home,xy,u,z,0)));
       break;
     default:
       throw UnknownRelation("Int::count");
