@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-03-03 17:32:21 +0100 (Wed, 03 Mar 2010) $ by $Author: schulte $
- *     $Revision: 10364 $
+ *     $Date: 2010-08-23 18:53:25 +0200 (Mon, 23 Aug 2010) $ by $Author: tack $
+ *     $Revision: 11358 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -48,11 +48,9 @@ namespace Gecode { namespace Int { namespace Dom {
   template<class View>
   ExecStatus
   ReRange<View>::post(Home home, View x, int min, int max, BoolView b) {
-    if (min > max) {
-      return ES_FAILED;
-    } else if (min == max) {
+    if (min == max) {
       return Rel::ReEqDomInt<View,BoolView>::post(home,x,min,b);
-    } else if ((max < x.min()) || (min > x.max())) {
+    } else if ((min > max) || (max < x.min()) || (min > x.max())) {
       GECODE_ME_CHECK(b.zero(home));
     } else if ((min <= x.min()) && (x.max() <= max)) {
       GECODE_ME_CHECK(b.one(home));
