@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2009-10-14 10:56:06 +0200 (Wed, 14 Oct 2009) $ by $Author: schulte $
- *     $Revision: 9902 $
+ *     $Date: 2010-03-03 17:32:21 +0100 (Wed, 03 Mar 2010) $ by $Author: schulte $
+ *     $Revision: 10364 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -86,29 +86,30 @@ namespace Gecode { namespace Int { namespace Dom {
   ReIntSet<View>::propagate(Space& home, const ModEventDelta&) {
     IntSetRanges i_is(is);
     if (b.one()) {
-      GECODE_ME_CHECK(x0.inter_r(home,i_is,false)); goto subsumed;
+      GECODE_ME_CHECK(x0.inter_r(home,i_is,false));
+      return home.ES_SUBSUMED(*this);
     }
     if (b.zero()) {
-      GECODE_ME_CHECK(x0.minus_r(home,i_is,false)); goto subsumed;
+      GECODE_ME_CHECK(x0.minus_r(home,i_is,false));
+      return home.ES_SUBSUMED(*this);
     }
 
     {
       ViewRanges<View> i_x(x0);
 
-      switch (Iter::Ranges::subsumes(i_is,i_x)) {
-      case Iter::Ranges::SS_SUBSUMED:
-        GECODE_ME_CHECK(b.one_none(home)); goto subsumed;
-      case Iter::Ranges::SS_EMPTY:
-        GECODE_ME_CHECK(b.zero_none(home)); goto subsumed;
-      case Iter::Ranges::SS_NONE:
+      switch (Iter::Ranges::compare(i_x,i_is)) {
+      case Iter::Ranges::CS_SUBSET:
+        GECODE_ME_CHECK(b.one_none(home));
+        return home.ES_SUBSUMED(*this);
+      case Iter::Ranges::CS_DISJOINT:
+        GECODE_ME_CHECK(b.zero_none(home));
+        return home.ES_SUBSUMED(*this);
+      case Iter::Ranges::CS_NONE:
         break;
       default: GECODE_NEVER;
       }
     }
-
     return ES_FIX;
-  subsumed:
-    return ES_SUBSUMED(*this,home);
   }
 
 }}}
