@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-05-28 20:54:33 +0200 (Fri, 28 May 2010) $ by $Author: schulte $
- *     $Revision: 10982 $
+ *     $Date: 2011-09-19 14:02:26 +0200 (Mon, 19 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12400 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -270,8 +270,8 @@ namespace Gecode { namespace Int { namespace Element {
     for (ViewValues<V0> i(x0); i(); ++i)
       if (c[i.val()] != x1.val())
         v[n++]=i.val();
-    Iter::Values::Array i(v,n);
-    GECODE_ME_CHECK(x0.minus_v(home,i,false));
+    Iter::Values::Array iv(v,n);
+    GECODE_ME_CHECK(x0.minus_v(home,iv,false));
     return ES_OK;
   }
 
