@@ -11,8 +11,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2010-09-03 16:36:31 +0200 (Fri, 03 Sep 2010) $ by $Author: schulte $
- *     $Revision: 11390 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -422,7 +422,7 @@ namespace Gecode { namespace Int { namespace Element {
   ViewBnd<VA,VB,VC>::propagate(Space& home, const ModEventDelta&) {
     assert(iv.size() > 1);
     RelTestBnd<VA,VC> rt;
-    GECODE_ME_CHECK((scan<VA,VB,VC,PC_INT_BND,RelTestBnd<VA,VC> >
+    GECODE_ES_CHECK((scan<VA,VB,VC,PC_INT_BND,RelTestBnd<VA,VC> >
                      (home,iv,x0,x1,*this,rt)));
     if (iv.size() == 1) {
       ExecStatus es = home.ES_SUBSUMED(*this);
@@ -513,7 +513,7 @@ namespace Gecode { namespace Int { namespace Element {
     assert(iv.size() > 1);
     if (VA::me(med) != ME_INT_DOM) {
       RelTestBnd<VA,VC> rt;
-      GECODE_ME_CHECK((scan<VA,VB,VC,PC_INT_DOM,RelTestBnd<VA,VC> >
+      GECODE_ES_CHECK((scan<VA,VB,VC,PC_INT_DOM,RelTestBnd<VA,VC> >
                        (home,iv,x0,x1,*this,rt)));
       if (iv.size() == 1) {
         ExecStatus es = home.ES_SUBSUMED(*this);
@@ -534,7 +534,7 @@ namespace Gecode { namespace Int { namespace Element {
         home.ES_NOFIX_PARTIAL(*this,VA::med(ME_INT_DOM));
     }
     RelTestDom<VA,VC> rt;
-    GECODE_ME_CHECK((scan<VA,VB,VC,PC_INT_DOM,RelTestDom<VA,VC> >
+    GECODE_ES_CHECK((scan<VA,VB,VC,PC_INT_DOM,RelTestDom<VA,VC> >
                      (home,iv,x0,x1,*this,rt)));
     if (iv.size() == 1) {
       ExecStatus es = home.ES_SUBSUMED(*this);
@@ -546,7 +546,7 @@ namespace Gecode { namespace Int { namespace Element {
     ViewRanges<VA>* i_view = r.alloc<ViewRanges<VA> >(iv.size());
     for (int i = iv.size(); i--; )
       i_view[i].init(iv[i].view);
-    Iter::Ranges::NaryUnion<ViewRanges<VA> > i_val(r, i_view, iv.size());
+    Iter::Ranges::NaryUnion i_val(r, i_view, iv.size());
     ModEvent me = x1.inter_r(home,i_val);
     r.free<ViewRanges<VA> >(i_view,iv.size());
     GECODE_ME_CHECK(me);
