@@ -7,8 +7,8 @@
  *     Christian Schulte, 2009
  *
  *  Last modified:
- *     $Date: 2009-10-12 17:36:53 +0200 (Mon, 12 Oct 2009) $ by $Author: schulte $
- *     $Revision: 9878 $
+ *     $Date: 2010-03-03 17:32:21 +0100 (Wed, 03 Mar 2010) $ by $Author: schulte $
+ *     $Revision: 10364 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -67,7 +67,7 @@ namespace Gecode { namespace Int { namespace Exec {
       t(home);
     }
     if (home.failed()) return ES_FAILED;
-    return ES_SUBSUMED(*this,sizeof(*this));
+    return home.ES_SUBSUMED(*this);
   }
 
 
