@@ -13,8 +13,8 @@
  *     Guido Tack, 2009
  *
  *  Last modified:
- *     $Date: 2010-07-14 17:46:18 +0200 (Wed, 14 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11192 $
+ *     $Date: 2011-09-01 15:04:29 +0200 (Thu, 01 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12384 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -1218,7 +1218,6 @@ namespace Gecode { namespace Int { namespace GCC {
         if (x[i].assigned()) {
           int  v = x[i].val();
           ValNode* rv = NULL;
-          int rv_idx  = 0;
           Edge* mub = vrn->get_match(UBC);
           if ((mub != NULL) && (v != mub->getVal()->val)) {
             mub->unmatch(UBC);
@@ -1244,7 +1243,6 @@ namespace Gecode { namespace Int { namespace GCC {
               e->unlink();
             } else {
               rv = e->getVal();
-              rv_idx = rv->kindex();
             }
           }
         } else {
