@@ -13,8 +13,8 @@
  *     Guido Tack, 2009
  *
  *  Last modified:
- *     $Date: 2010-09-03 16:36:31 +0200 (Fri, 03 Sep 2010) $ by $Author: schulte $
- *     $Revision: 11390 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -147,8 +147,7 @@ namespace Gecode { namespace Int { namespace GCC {
         ViewRanges<IntView> iter(x[i]);
         xrange[i] = iter;
       }
-      Iter::Ranges::NaryUnion<ViewRanges<IntView> > 
-        drl(r, &xrange[0], x.size());
+      Iter::Ranges::NaryUnion drl(r, &xrange[0], x.size());
       if (static_cast<unsigned int>(k.size()) == Iter::Ranges::size(drl)) {
         for (int i=k.size(); i--;)
           if (k[i].min() != 1 || k[i].max() != 1)
