@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2010-08-02 14:34:35 +0200 (Mon, 02 Aug 2010) $ by $Author: tack $
- *     $Revision: 11321 $
+ *     $Date: 2011-09-02 11:52:00 +0200 (Fri, 02 Sep 2011) $ by $Author: tack $
+ *     $Revision: 12388 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -87,18 +87,24 @@ namespace Gecode { namespace Int { namespace Linear {
     for (int i = n_p; i--; ) {
       sl += t_p[i].a * static_cast<double>(t_p[i].x.min());
       su += t_p[i].a * static_cast<double>(t_p[i].x.max());
+      if ((sl < Limits::double_min) || (su > Limits::double_max))
+        throw OutOfLimits("Int::linear");
     }
     for (int i = n_n; i--; ) {
       sl -= t_n[i].a * static_cast<double>(t_n[i].x.max());
       su -= t_n[i].a * static_cast<double>(t_n[i].x.min());
+      if ((sl < Limits::double_min) || (su > Limits::double_max))
+        throw OutOfLimits("Int::linear");
     }
+
+    bool is_ip = (sl >= Limits::min) && (su <= Limits::max);
+
     sl -= d;
     su -= d;
-
     if ((sl < Limits::double_min) || (su > Limits::double_max))
       throw OutOfLimits("Int::linear");
 
-    bool is_ip = (sl >= Limits::min) && (su <= Limits::max);
+    is_ip = is_ip && (sl >= Limits::min) && (su <= Limits::max);
 
     for (int i = n_p; i--; ) {
       if (sl - t_p[i].a * static_cast<double>(t_p[i].x.min()) 
