@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2010-07-13 22:35:02 +0200 (Tue, 13 Jul 2010) $ by $Author: tack $
- *     $Revision: 11183 $
+ *     $Date: 2011-08-12 15:27:20 +0200 (Fri, 12 Aug 2011) $ by $Author: tack $
+ *     $Revision: 12285 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -45,14 +45,16 @@ namespace Gecode { namespace Int { namespace Linear {
   estimate(Term<View>* t, int n, int c, int& l, int &u) {
     double min = c;
     double max = c;
-    for (int i=n; i--; )
-      if (t[i].a > 0) {
-        min += t[i].a*t[i].x.min();
-        max += t[i].a*t[i].x.max();
+    for (int i=n; i--; ) {
+      double a = t[i].a;
+      if (a > 0) {
+        min += a*t[i].x.min();
+        max += a*t[i].x.max();
       } else {
-        max += t[i].a*t[i].x.min();
-        min += t[i].a*t[i].x.max();
+        max += a*t[i].x.min();
+        min += a*t[i].x.max();
       }
+    }
     if (min < Limits::min)
       min = Limits::min;
     if (min > Limits::max)
