@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-28 16:13:53 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11292 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -309,23 +309,21 @@ namespace Gecode { namespace Int { namespace Rel {
       ViewRanges<View> i_xi(x[i]);
       i_x[i] = i_xi;
     }
-    Iter::Ranges::NaryInter<ViewRanges<View> > r(i_x,n);
-    Iter::Ranges::Cache<Iter::Ranges::NaryInter<ViewRanges<View> > > 
-      rc(re,r);
+    Iter::Ranges::NaryInter r(re,i_x,n);
 
-    if (!rc())
+    if (!r())
       return ES_FAILED;
-    ++rc;
-    if (!rc()) {
-      rc.reset();
+    ++r;
+    if (!r()) {
+      r.reset();
       for (int i = n; i--; ) {
-        GECODE_ME_CHECK(x[i].gq(home,rc.min()));
-        GECODE_ME_CHECK(x[i].lq(home,rc.max()));
+        GECODE_ME_CHECK(x[i].gq(home,r.min()));
+        GECODE_ME_CHECK(x[i].lq(home,r.max()));
       }
     } else {
       for (int i = n; i--; ) {
-        rc.reset();
-        GECODE_ME_CHECK(x[i].narrow_r(home,rc,false));
+        r.reset();
+        GECODE_ME_CHECK(x[i].narrow_r(home,r,false));
       }
     }
     return ES_FIX;
@@ -346,6 +344,7 @@ namespace Gecode { namespace Int { namespace Rel {
   template<class View>
   ExecStatus
   NaryEqBnd<View>::post(Home home, ViewArray<View>& x) {
+    x.unique(home);
     if (x.size() == 2) {
       return EqBnd<View,View>::post(home,x[0],x[1]);
     } else if (x.size() > 2) {
