@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2010-06-29 10:39:13 +0200 (Tue, 29 Jun 2010) $ by $Author: schulte $
- *     $Revision: 11118 $
+ *     $Date: 2010-08-31 17:19:33 +0200 (Tue, 31 Aug 2010) $ by $Author: schulte $
+ *     $Revision: 11368 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -45,10 +45,10 @@ namespace Gecode { namespace Int {
   BoolView::BoolView(void) {}
   forceinline
   BoolView::BoolView(const BoolVar& y)
-    : VarImpView<BoolVarImp>(y.varimp()) {}
+    : VarImpView<BoolVar>(y.varimp()) {}
   forceinline
   BoolView::BoolView(BoolVarImp* y)
-    : VarImpView<BoolVarImp>(y) {}
+    : VarImpView<BoolVar>(y) {}
 
   /*
    * Value access
@@ -177,37 +177,31 @@ namespace Gecode { namespace Int {
   template<class I>
   forceinline ModEvent
   BoolView::narrow_r(Space& home, I& i, bool depend) {
-    Iter::Ranges::IsRangeIter<I>();
     return x->narrow_r(home,i,depend);
   }
   template<class I>
   forceinline ModEvent
   BoolView::inter_r(Space& home, I& i, bool depend) {
-    Iter::Ranges::IsRangeIter<I>();
     return x->inter_r(home,i,depend);
   }
   template<class I>
   forceinline ModEvent
   BoolView::minus_r(Space& home, I& i, bool depend) {
-    Iter::Ranges::IsRangeIter<I>();
     return x->minus_r(home,i,depend);
   }
   template<class I>
   forceinline ModEvent
   BoolView::narrow_v(Space& home, I& i, bool depend) {
-    Iter::Values::IsValueIter<I>();
     return x->narrow_v(home,i,depend);
   }
   template<class I>
   forceinline ModEvent
   BoolView::inter_v(Space& home, I& i, bool depend) {
-    Iter::Values::IsValueIter<I>();
     return x->inter_v(home,i,depend);
   }
   template<class I>
   forceinline ModEvent
   BoolView::minus_v(Space& home, I& i, bool depend) {
-    Iter::Values::IsValueIter<I>();
     return x->minus_v(home,i,depend);
   }
 
@@ -282,7 +276,7 @@ namespace Gecode { namespace Int {
 
   forceinline ModEventDelta
   BoolView::med(ModEvent me) {
-    return VarImpView<BoolVarImp>::med(me);
+    return VarImpView<BoolVar>::med(me);
   }
 
   /**
