@@ -7,8 +7,8 @@
  *     Christian Schulte, 2003
  *
  *  Last modified:
- *     $Date: 2010-06-29 10:39:13 +0200 (Tue, 29 Jun 2010) $ by $Author: schulte $
- *     $Revision: 11118 $
+ *     $Date: 2010-07-28 17:35:33 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
+ *     $Revision: 11294 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -170,13 +170,11 @@ namespace Gecode { namespace Int {
   template<class I>
   forceinline ModEvent
   ConstIntView::narrow_r(Space&, I& i, bool) {
-    Iter::Ranges::IsRangeIter<I>();
     return i() ? ME_INT_NONE : ME_INT_FAILED;
   }
   template<class I>
   forceinline ModEvent
   ConstIntView::inter_r(Space&, I& i, bool) {
-    Iter::Ranges::IsRangeIter<I>();
     while (i() && (i.max() < x))
       ++i;
     return (i() && (i.min() <= x)) ? ME_INT_NONE : ME_INT_FAILED;
@@ -184,7 +182,6 @@ namespace Gecode { namespace Int {
   template<class I>
   forceinline ModEvent
   ConstIntView::minus_r(Space&, I& i, bool) {
-    Iter::Ranges::IsRangeIter<I>();
     while (i() && (i.max() < x))
       ++i;
     return (i() && (i.min() <= x)) ? ME_INT_FAILED : ME_INT_NONE;
@@ -192,13 +189,11 @@ namespace Gecode { namespace Int {
   template<class I>
   forceinline ModEvent
   ConstIntView::narrow_v(Space&, I& i, bool) {
-    Iter::Values::IsValueIter<I>();
     return i() ? ME_INT_NONE : ME_INT_FAILED;
   }
   template<class I>
   forceinline ModEvent
   ConstIntView::inter_v(Space&, I& i, bool) {
-    Iter::Values::IsValueIter<I>();
     while (i() && (i.val() < x))
       ++i;
     return (i() && (i.val() == x)) ? ME_INT_NONE : ME_INT_FAILED;
@@ -206,7 +201,6 @@ namespace Gecode { namespace Int {
   template<class I>
   forceinline ModEvent
   ConstIntView::minus_v(Space&, I& i, bool) {
-    Iter::Values::IsValueIter<I>();
     while (i() && (i.val() < x))
       ++i;
     return (i() && (i.val() == x)) ? ME_INT_FAILED : ME_INT_NONE;
