@@ -7,8 +7,8 @@
  *     Christian Schulte, 2003
  *
  *  Last modified:
- *     $Date: 2010-07-28 16:13:53 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11292 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -163,42 +163,42 @@ namespace Gecode { namespace Int {
   forceinline ModEvent
   MinusView::narrow_r(Space& home, I& i, bool) {
     Region r(home);
-    Iter::Ranges::Minus<I> mi(r,i);
+    Iter::Ranges::Minus mi(r,i);
     return x.narrow_r(home,mi,false);
   }
   template<class I>
   forceinline ModEvent
   MinusView::inter_r(Space& home, I& i, bool) {
     Region r(home);
-    Iter::Ranges::Minus<I> mi(r,i);
+    Iter::Ranges::Minus mi(r,i);
     return x.inter_r(home,mi,false);
   }
   template<class I>
   forceinline ModEvent
   MinusView::minus_r(Space& home, I& i, bool) {
     Region r(home);
-    Iter::Ranges::Minus<I> mi(r,i);
+    Iter::Ranges::Minus mi(r,i);
     return x.minus_r(home,mi,false);
   }
   template<class I>
   forceinline ModEvent
   MinusView::narrow_v(Space& home, I& i, bool) {
     Region r(home);
-    Iter::Values::Minus<I> mi(r,i);
+    Iter::Values::Minus mi(r,i);
     return x.narrow_v(home,mi,false);
   }
   template<class I>
   forceinline ModEvent
   MinusView::inter_v(Space& home, I& i, bool) {
     Region r(home);
-    Iter::Values::Minus<I> mi(r,i);
+    Iter::Values::Minus mi(r,i);
     return x.inter_v(home,mi,false);
   }
   template<class I>
   forceinline ModEvent
   MinusView::minus_v(Space& home, I& i, bool) {
     Region r(home);
-    Iter::Values::Minus<I> mi(r,i);
+    Iter::Values::Minus mi(r,i);
     return x.minus_v(home,mi,false);
   }
 
