@@ -7,8 +7,8 @@
  *     Christian Schulte, 2003
  *
  *  Last modified:
- *     $Date: 2010-04-28 18:54:51 +0200 (Wed, 28 Apr 2010) $ by $Author: tack $
- *     $Revision: 10822 $
+ *     $Date: 2011-09-28 14:14:32 +0200 (Wed, 28 Sep 2011) $ by $Author: tack $
+ *     $Revision: 12417 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -107,16 +107,22 @@ namespace Gecode { namespace Int {
   operator <<(std::basic_ostream<Char,Traits>& os, const OffsetView& x) {
     return print_view(os,x);
   }
+  template<class Char, class Traits, class View>
+  inline std::basic_ostream<Char,Traits>&
+  operator <<(std::basic_ostream<Char,Traits>& os,
+              const CachedView<View>& x) {
+    return print_view(os,x);
+  }
 
   template<class Char, class Traits>
   inline std::basic_ostream<Char,Traits>&
   operator <<(std::basic_ostream<Char,Traits>& os, const IntScaleView& x) {
-    return print_scale<int,unsigned int>(os,x);
+    return print_scale<Char,Traits,int,unsigned int>(os,x);
   }
   template<class Char, class Traits>
   inline std::basic_ostream<Char,Traits>&
   operator <<(std::basic_ostream<Char,Traits>& os, const DoubleScaleView& x) {
-    return print_scale<double,double>(os,x);
+    return print_scale<Char,Traits,double,double>(os,x);
   }
 
   template<class Char, class Traits>
