@@ -7,8 +7,8 @@
  *     Christian Schulte, 2005
  *
  *  Last modified:
- *     $Date: 2009-09-08 21:10:29 +0200 (Tue, 08 Sep 2009) $ by $Author: schulte $
- *     $Revision: 9692 $
+ *     $Date: 2010-07-28 17:35:33 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
+ *     $Revision: 11294 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -51,9 +51,6 @@ namespace Gecode { namespace Iter { namespace Ranges {
    */
   template<class Val, class UnsVal, class I>
   class ScaleUp {
-  private:
-    /// Check that \a I is a range iterator;
-    IsRangeIter<I> _checkI;
   protected:
     /// Iterator to be scaled
     I i;
@@ -100,9 +97,6 @@ namespace Gecode { namespace Iter { namespace Ranges {
    */
   template<class I>
   class ScaleDown : public MinMax {
-  private:
-    /// Check that \a I is a range iterator;
-    IsRangeIter<I> _checkI;
   protected:
     /// Iterator to be scaled down
     I i;
