@@ -13,8 +13,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2011-01-27 13:03:05 +0100 (Thu, 27 Jan 2011) $ by $Author: schulte $
- *     $Revision: 11570 $
+ *     $Date: 2011-08-18 12:10:04 +0200 (Thu, 18 Aug 2011) $ by $Author: tack $
+ *     $Revision: 12313 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -273,10 +273,14 @@ namespace Gecode {
     ViewArray(void);
     /// Allocate array with \a m views
     ViewArray(Space& home, int m);
+    /// Allocate array with \a m views
+    ViewArray(Region& r, int m);
     /// Initialize from view array \a a (share elements)
     ViewArray(const ViewArray<View>& a);
     /// Initialize from view array \a a (copy elements)
     ViewArray(Space& home, const ViewArray<View>& a);
+    /// Initialize from view array \a a (copy elements)
+    ViewArray(Region& r, const ViewArray<View>& a);
     /// Initialize from view array \a a (share elements)
     const ViewArray<View>& operator =(const ViewArray<View>& a);
     /**
@@ -297,6 +301,24 @@ namespace Gecode {
         x = NULL;
       }
     }
+    /**
+     * \brief Initialize from variable argument array \a a (copy elements)
+     *
+     * Note that the view type \a View must provide a constructor
+     * for the associated \a Var type.
+     */
+    template<class Var>
+    ViewArray(Region& r, const VarArgArray<Var>& a)
+      : n(a.size()) {
+      // This may not be in the hpp file (to satisfy the MS compiler)
+      if (n>0) {
+        x = r.alloc<View>(n);
+        for (int i=n; i--; )
+          x[i]=a[i];
+      } else {
+        x = NULL;
+      }
+    }
     //@}
 
     /// \name Array size
@@ -963,8 +985,8 @@ namespace Gecode {
   template<class Var>
   typename ArrayTraits<VarArgArray<Var> >::ArgsType
   VarArray<Var>::slice(int start, int inc, int maxN) {
-    assert(start < n);
-    if (maxN<0)
+    assert(n==0 || start < n);
+    if (n==0 || maxN<0)
       maxN = n;
     int s;
     if (inc == 0)
@@ -1128,6 +1150,12 @@ namespace Gecode {
     : n(n0) {
     x = (n>0) ? home.alloc<View>(n) : NULL;
   }
+  template<class View>
+  forceinline
+  ViewArray<View>::ViewArray(Region& r, int n0)
+    : n(n0) {
+    x = (n>0) ? r.alloc<View>(n) : NULL;
+  }
 
   template<class View>
   ViewArray<View>::ViewArray(Space& home, const ViewArray<View>& a)
@@ -1140,6 +1168,17 @@ namespace Gecode {
       x = NULL;
     }
   }
+  template<class View>
+  ViewArray<View>::ViewArray(Region& r, const ViewArray<View>& a)
+    : n(a.size()) {
+    if (n>0) {
+      x = r.alloc<View>(n);
+      for (int i = n; i--; )
+        x[i] = a[i];
+    } else {
+      x = NULL;
+    }
+  }
 
   template<class View>
   forceinline
@@ -1663,8 +1702,8 @@ namespace Gecode {
   template<class T> template<class A>
   A
   ArgArrayBase<T>::slice(int start, int inc, int maxN) {
-    assert(start < n);
-    if (maxN<0)
+    assert(n==0 || start < n);
+    if (n==0 || maxN<0)
       maxN = n;
     int s;
     if (inc == 0)
