@@ -9,8 +9,8 @@
  *     Mikael Lagerkvist, 2008
  *
  *  Last modified:
- *     $Date: 2010-04-08 12:35:31 +0200 (Thu, 08 Apr 2010) $ by $Author: schulte $
- *     $Revision: 10684 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -62,6 +62,10 @@ namespace Gecode {
       Description(const Brancher& b, unsigned int a) : Choice(b,a) {}
       /// Report size occupied
       virtual size_t size(void) const { return sizeof(Description); }
+      /// Archive into \a e
+      virtual void archive(Archive& e) const {
+        Choice::archive(e);
+      }
     };
     /// Function to call
     void (*f)(Space&);
@@ -83,6 +87,10 @@ namespace Gecode {
       assert(!done);
       return new Description(*this,1);
     }
+    /// Return choice
+    virtual const Choice* choice(const Space&, Archive&) {
+      return new Description(*this,1);
+    }
     /// Perform commit
     virtual ExecStatus 
     commit(Space& home, const Choice&, unsigned int) {
