@@ -7,8 +7,8 @@
  *     Christian Schulte, 2008
  *
  *  Last modified:
- *     $Date: 2010-04-08 12:35:31 +0200 (Thu, 08 Apr 2010) $ by $Author: schulte $
- *     $Revision: 10684 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -67,6 +67,8 @@ namespace Gecode {
       Choice(const typename A::Choice& a, const typename B::Choice& b);
       /// Report size occupied
       size_t size(void) const;
+      /// Archive into \a e
+      void archive(Archive& e) const;
     };
     /// Default constructor
     ViewSelTieBreakStatic(void);
@@ -78,6 +80,8 @@ namespace Gecode {
     ViewSelStatus select(Space& home, typename A::View x);
     /// Return choice
     Choice choice(Space& home);
+    /// Return choice
+    Choice choice(const Space& home, Archive& e);
     /// Commit to choice
     void commit(Space& home, const Choice& c, unsigned int a);
     /// Updating during cloning
@@ -97,6 +101,8 @@ namespace Gecode {
     virtual size_t size(void) const = 0;
     /// Destructor
     GECODE_KERNEL_EXPORT virtual ~ChoiceVirtualBase(void);
+    /// Archive into \a e
+    virtual void archive(Archive& e) const = 0;
     /// \name Memory management
     //@{
     /// Allocate memory
@@ -118,6 +124,8 @@ namespace Gecode {
     virtual ViewSelStatus select(Space& home, View x) = 0;
     /// Return choice
     virtual ChoiceVirtualBase* choice(Space& home) = 0;
+    /// Return choice
+    virtual ChoiceVirtualBase* choice(const Space& home, Archive& e) = 0;
     /// Commit to choice
     virtual void commit(Space& home, const ChoiceVirtualBase* c, 
                         unsigned int a) = 0;
@@ -152,6 +160,8 @@ namespace Gecode {
     virtual size_t size(void) const;
     /// Destructor
     virtual ~ChoiceVirtual(void);
+    /// Archive into \a e
+    virtual void archive(Archive& e) const;
   };
 
   /**
@@ -173,6 +183,8 @@ namespace Gecode {
     virtual ViewSelStatus select(Space& home, typename ViewSel::View x);
     /// Return choice
     virtual ChoiceVirtualBase* choice(Space& home);
+    /// Return choice
+    virtual ChoiceVirtualBase* choice(const Space& home, Archive& e);
     /// Commit to choice
     virtual void commit(Space& home, const ChoiceVirtualBase* d,
                         unsigned int a);
@@ -205,6 +217,9 @@ namespace Gecode {
       ChoiceVirtualBase** c;
       /// Constructor
       Choice(Space& home, ViewSelVirtualBase<_View>** tb, int n0);
+      /// Constructor
+      Choice(const Space& home, Archive& e,
+             ViewSelVirtualBase<_View>** tb, int n0);
       /// Copy constructor
       Choice(const Choice& ce);
       /// Assignment operator
@@ -216,6 +231,8 @@ namespace Gecode {
       size_t size(void) const;
       /// Destructor
       ~Choice(void);
+      /// Archive into \a e
+      void archive(Archive& e) const;
     };
     /// Default constructor
     ViewSelTieBreakDynamic(void);
@@ -228,6 +245,8 @@ namespace Gecode {
     ViewSelStatus select(Space& home, _View x);
     /// Return choice
     Choice choice(Space& home);
+    /// Return choice
+    Choice choice(const Space& home, Archive& e);
     /// Commit to choice
     void commit(Space& home,
                 const typename ViewSelTieBreakDynamic<_View>::Choice& c,
@@ -251,6 +270,12 @@ namespace Gecode {
   ViewSelTieBreakStatic<A,B>::Choice::size(void) const {
     return a.size() + b.size();
   }
+  template<class A, class B>
+  forceinline void
+  ViewSelTieBreakStatic<A,B>::Choice::archive(Archive& e) const {
+    a.archive(e);
+    b.archive(e);
+  }
 
   template<class A, class B>
   forceinline
@@ -297,6 +322,13 @@ namespace Gecode {
     return c;
   }
   template<class A, class B>
+  inline typename ViewSelTieBreakStatic<A,B>::Choice
+  ViewSelTieBreakStatic<A,B>::choice(const Space& home, Archive& e) {
+    typename ViewSelTieBreakStatic<A,B>::Choice c(a.choice(home,e),
+                                                  b.choice(home,e));
+    return c;
+  }
+  template<class A, class B>
   forceinline void
   ViewSelTieBreakStatic<A,B>::commit(Space& home, const Choice& c,
                                      unsigned int al) {
@@ -361,7 +393,10 @@ namespace Gecode {
   }
   template<class Choice>
   ChoiceVirtual<Choice>::~ChoiceVirtual(void) {}
-
+  template<class Choice> void
+  ChoiceVirtual<Choice>::archive(Archive& e) const {
+    choice.archive(e);
+  }
 
   template<class ViewSel>
   forceinline
@@ -390,6 +425,11 @@ namespace Gecode {
     return new ChoiceVirtual<typename ViewSel::Choice>(viewsel.choice(home));
   }
   template<class ViewSel>
+  ChoiceVirtualBase*
+  ViewSelVirtual<ViewSel>::choice(const Space& home, Archive& e) {
+    return new ChoiceVirtual<typename ViewSel::Choice>(viewsel.choice(home,e));
+  }
+  template<class ViewSel>
   void
   ViewSelVirtual<ViewSel>::commit(Space& home, const ChoiceVirtualBase* _c,
                                   unsigned int a) {
@@ -421,6 +461,15 @@ namespace Gecode {
   }
   template<class View>
   forceinline
+  ViewSelTieBreakDynamic<View>::Choice::Choice
+  (const Space& home, Archive& e, ViewSelVirtualBase<View>** tb,
+   int n0)
+    : n(n0), c(heap.alloc<ChoiceVirtualBase*>(n)) {
+    for (int i=n; i--; )
+      c[i] = tb[i]->choice(home,e);
+  }
+  template<class View>
+  forceinline
   ViewSelTieBreakDynamic<View>::Choice::Choice(const Choice& ce)
     : n(ce.n), c(heap.alloc<ChoiceVirtualBase*>(n)) {
     for (int i=n; i--; )
@@ -460,6 +509,13 @@ namespace Gecode {
       delete c[i];
     heap.free(c,n);
   }
+  template<class View>
+  forceinline void
+  ViewSelTieBreakDynamic<View>::Choice::archive(Archive& e) const 
+  {
+    for (int i=0; i<n; i++)
+      c[i]->archive(e);
+  }
 
 
   // Select variable with dynamic tie breaking
@@ -525,6 +581,12 @@ namespace Gecode {
     return c;
   }
   template<class _View>
+  inline typename ViewSelTieBreakDynamic<_View>::Choice
+  ViewSelTieBreakDynamic<_View>::choice(const Space& home, Archive& e) {
+    Choice c(home,e,tb,n);
+    return c;
+  }
+  template<class _View>
   forceinline void
   ViewSelTieBreakDynamic<_View>::commit
   (Space& home, const typename ViewSelTieBreakDynamic<_View>::Choice& c,
