@@ -9,8 +9,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2010-09-01 12:35:28 +0200 (Wed, 01 Sep 2010) $ by $Author: zayenz $
- *     $Revision: 11372 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -124,6 +124,8 @@ namespace Gecode {
   public:
     /// Return choice
     virtual const Choice* choice(Space& home);
+    /// Return choice
+    virtual const Choice* choice(const Space& home, Archive& e);
     /// Perform commit for choice \a c and alternative \a a
     virtual ExecStatus commit(Space& home, const Choice& c, unsigned int a);
     /// Perform cloning
@@ -155,6 +157,8 @@ namespace Gecode {
     const typename ViewSel::Choice& viewchoice(void) const;
     /// Report size occupied
     virtual size_t size(void) const;
+    /// Archive into \a e
+    virtual void archive(Archive& e) const;
   };
 
   /// %Choice storing position and value
@@ -177,6 +181,8 @@ namespace Gecode {
     const typename ValSel::Val& val(void) const;
     /// Report size occupied
     virtual size_t size(void) const;
+    /// Archive into \a e
+    virtual void archive(Archive& e) const;
   };
   //@}
 
@@ -216,7 +222,13 @@ namespace Gecode {
   PosChoice<ViewSel>::size(void) const {
     return sizeof(PosChoice<ViewSel>) + _viewchoice.size();
   }
-
+  template<class ViewSel>
+  forceinline void
+  PosChoice<ViewSel>::archive(Archive& e) const {
+    Choice::archive(e);
+    e << _pos.pos;
+    _viewchoice.archive(e);
+  }
 
   /*
    * %Choice with position and value
@@ -250,6 +262,13 @@ namespace Gecode {
     return sizeof(PosValChoice<ViewSel,ValSel>) + _valchoice.size();
   }
 
+  template<class ViewSel, class ValSel>
+  forceinline void
+  PosValChoice<ViewSel, ValSel>::archive(Archive& e) const {
+    PosChoice<ViewSel>::archive(e);
+    _valchoice.archive(e);
+    e << _val;
+  }
 
   /*
    * Generic brancher based on view selection
@@ -376,6 +395,16 @@ namespace Gecode {
        viewsel.choice(home),valsel.choice(home),val);
   }
 
+  template<class ViewSel, class ValSel>
+  const Choice*
+  ViewValBrancher<ViewSel,ValSel>::choice(const Space& home, Archive& e) {
+    int p; e >> p;
+    typename ViewSel::Choice viewsc = viewsel.choice(home,e);
+    typename ValSel::Choice valsc = valsel.choice(home,e);
+    typename ValSel::Val val; e >> val;
+    return new PosValChoice<ViewSel,ValSel>(*this,p,viewsc,valsc,val);
+  }
+
   template<class ViewSel, class ValSel>
   ExecStatus
   ViewValBrancher<ViewSel,ValSel>
