@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2010-07-14 12:28:41 +0200 (Wed, 14 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11184 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -334,6 +334,18 @@ namespace Gecode {
     return b_status->choice(*this);
   }
 
+  const Choice*
+  Space::choice(Archive& e) const {
+    unsigned int id; e >> id;
+    Brancher* b_cur = Brancher::cast(bl.next());
+    while (b_cur != Brancher::cast(&bl)) {
+      if (id == b_cur->id())
+        return b_cur->choice(*this,e);
+      b_cur = Brancher::cast(b_cur->next());
+    }
+    throw SpaceNoBrancher();
+  }
+
   void
   Space::_commit(const Choice& c, unsigned int a) {
     if (a >= c.alternatives())
@@ -479,7 +491,7 @@ namespace Gecode {
       static_cast<VarImp<NoIdxVarImpConf>*>(c->pc.c.vars_noidx);
     while (x != NULL) {
       VarImp<NoIdxVarImpConf>* n = x->next();
-      x->base = NULL; x->u.idx[0] = 0;
+      x->b.base = NULL; x->u.idx[0] = 0;
       x = n;
     }
     // Update variables with indexing structure
@@ -541,6 +553,10 @@ namespace Gecode {
     home.pc.c.local = this;
   }
 
+  void
+  Choice::archive(Archive& e) const {
+    e << id();
+  }
 
 }
 
