@@ -7,8 +7,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-16 10:51:05 +0200 (Fri, 16 Jul 2010) $ by $Author: tack $
- *     $Revision: 11208 $
+ *     $Date: 2011-01-21 16:01:14 +0100 (Fri, 21 Jan 2011) $ by $Author: schulte $
+ *     $Revision: 11553 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -44,6 +44,9 @@ namespace Gecode {
   forceinline
   BoolExpr::Node::Node(void) : use(1) {}
 
+  forceinline
+  BoolExpr::Node::~Node(void) { delete m; }
+
   forceinline void*
   BoolExpr::Node::operator new(size_t size) {
     return heap.ralloc(size);
@@ -53,82 +56,23 @@ namespace Gecode {
     heap.rfree(p);
   }
 
+  forceinline void*
+  BoolExpr::MiscExpr::operator new(size_t size) {
+    return heap.ralloc(size);
+  }
+  forceinline void
+  BoolExpr::MiscExpr::operator delete(void* p, size_t) {
+    heap.rfree(p);
+  }
+
+  forceinline
+  BoolExpr::BoolExpr(void) : n(NULL) {}
+
   forceinline
   BoolExpr::BoolExpr(const BoolExpr& e) : n(e.n) {
     n->use++;
   }
 
-  forceinline
-  BoolExpr::BoolExpr(const BoolVar& x) : n(new Node) {
-    n->same = 1;
-    n->t    = NT_VAR;
-    n->l    = NULL;
-    n->r    = NULL;
-    n->x    = x;
-  }
-
-  forceinline
-  BoolExpr::BoolExpr(const BoolExpr& l, NodeType t, const BoolExpr& r)
-    : n(new Node) {
-    int ls = ((l.n->t == t) || (l.n->t == NT_VAR)) ? l.n->same : 1;
-    int rs = ((r.n->t == t) || (r.n->t == NT_VAR)) ? r.n->same : 1;
-    n->same = ls+rs;
-    n->t    = t;
-    n->l    = l.n;
-    n->l->use++;
-    n->r    = r.n;
-    n->r->use++;
-  }
-
-  forceinline
-  BoolExpr::BoolExpr(const BoolExpr& l, NodeType t) {
-    (void) t;
-    assert(t == NT_NOT);
-    if (l.n->t == NT_NOT) {
-      n = l.n->l;
-      n->use++;
-    } else {
-      n = new Node;
-      n->same = 1;
-      n->t    = NT_NOT;
-      n->l    = l.n;
-      n->l->use++;
-      n->r    = NULL;
-    }
-  }
-
-  forceinline
-  BoolExpr::BoolExpr(const LinRel& rl)
-    : n(new Node) {
-    n->same = 1;
-    n->t    = NT_RLIN;
-    n->l    = NULL;
-    n->r    = NULL;
-    n->rl   = rl;
-  }
-
-#ifdef GECODE_HAS_SET_VARS
-  forceinline
-  BoolExpr::BoolExpr(const SetRel& rs)
-    : n(new Node) {
-    n->same = 1;
-    n->t    = NT_RSET;
-    n->l    = NULL;
-    n->r    = NULL;
-    n->rs   = rs;
-  }
-
-  forceinline
-  BoolExpr::BoolExpr(const SetCmpRel& rs)
-    : n(new Node) {
-    n->same = 1;
-    n->t    = NT_RSET;
-    n->l    = NULL;
-    n->r    = NULL;
-    n->rs   = rs;
-  }
-#endif
-
   inline BoolVar
   BoolExpr::expr(Home home, IntConLevel icl) const {
     Region r(home);
