@@ -11,8 +11,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-14 20:32:01 +0200 (Wed, 14 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11195 $
+ *     $Date: 2011-01-21 16:01:14 +0100 (Fri, 21 Jan 2011) $ by $Author: schulte $
+ *     $Revision: 11553 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -82,165 +82,12 @@ namespace Gecode {
    * Operations for expressions
    *
    */
-  forceinline
-  LinExpr::LinExpr(void) :
-    n(new Node) {
-    n->n_int = n->n_bool = 0;
-    n->t = NT_VAR_INT;
-    n->l = n->r = NULL;
-    n->a = 0;
-  }
-
-  forceinline
-  LinExpr::LinExpr(double c) :
-    n(new Node) {
-    n->n_int = n->n_bool = 0;
-    n->t = NT_CONST;
-    n->l = n->r = NULL;
-    n->a = 0;
-    Int::Limits::check(c,"MiniModel::LinExpr");
-    n->c = static_cast<int>(c);
-  }
-
   forceinline
   LinExpr::LinExpr(const LinExpr& e)
     : n(e.n) {
     n->use++;
   }
 
-  forceinline
-  LinExpr::LinExpr(const IntVar& x, int a) :
-    n(new Node) {
-    n->n_int = 1;
-    n->n_bool = 0;
-    n->t = NT_VAR_INT;
-    n->l = n->r = NULL;
-    n->a = a;
-    n->x_int = x;
-  }
-
-  forceinline
-  LinExpr::LinExpr(const BoolVar& x, int a) :
-    n(new Node) {
-    n->n_int = 0;
-    n->n_bool = 1;
-    n->t = NT_VAR_BOOL;
-    n->l = n->r = NULL;
-    n->a = a;
-    n->x_bool = x;
-  }
-
-  forceinline
-  LinExpr::LinExpr(const IntVarArgs& x) :
-    n(new Node) {
-    n->n_int = x.size();
-    n->n_bool = 0;
-    n->t = NT_SUM_INT;
-    n->l = n->r = NULL;
-    if (x.size() > 0) {
-      n->sum.ti = heap.alloc<Int::Linear::Term<Int::IntView> >(x.size());
-      for (int i=x.size(); i--; ) {
-        n->sum.ti[i].x = x[i];
-        n->sum.ti[i].a = 1;
-      }
-    }
-  }
-
-  forceinline
-  LinExpr::LinExpr(const IntArgs& a, const IntVarArgs& x) :
-    n(new Node) {
-    if (a.size() != x.size())
-      throw Int::ArgumentSizeMismatch("MiniModel::LinExpr");
-    n->n_int = x.size();
-    n->n_bool = 0;
-    n->t = NT_SUM_INT;
-    n->l = n->r = NULL;
-    if (x.size() > 0) {
-      n->sum.ti = heap.alloc<Int::Linear::Term<Int::IntView> >(x.size());
-      for (int i=x.size(); i--; ) {
-        n->sum.ti[i].x = x[i];
-        n->sum.ti[i].a = a[i];
-      }
-    }
-  }
-
-  forceinline
-  LinExpr::LinExpr(const BoolVarArgs& x) :
-    n(new Node) {
-    n->n_int = 0;
-    n->n_bool = x.size();
-    n->t = NT_SUM_BOOL;
-    n->l = n->r = NULL;
-    if (x.size() > 0) {
-      n->sum.tb = heap.alloc<Int::Linear::Term<Int::BoolView> >(x.size());
-      for (int i=x.size(); i--; ) {
-        n->sum.tb[i].x = x[i];
-        n->sum.tb[i].a = 1;
-      }
-    }
-  }
-
-  forceinline
-  LinExpr::LinExpr(const IntArgs& a, const BoolVarArgs& x) :
-    n(new Node) {
-    if (a.size() != x.size())
-      throw Int::ArgumentSizeMismatch("MiniModel::LinExpr");
-    n->n_int = 0;
-    n->n_bool = x.size();
-    n->t = NT_SUM_BOOL;
-    n->l = n->r = NULL;
-    if (x.size() > 0) {
-      n->sum.tb = heap.alloc<Int::Linear::Term<Int::BoolView> >(x.size());
-      for (int i=x.size(); i--; ) {
-        n->sum.tb[i].x = x[i];
-        n->sum.tb[i].a = a[i];
-      }
-    }
-  }
-
-  forceinline
-  LinExpr::LinExpr(const LinExpr& e0, NodeType t, const LinExpr& e1) :
-    n(new Node) {
-    n->n_int = e0.n->n_int + e1.n->n_int;
-    n->n_bool = e0.n->n_bool + e1.n->n_bool;
-    n->t = t;
-    n->l = e0.n; n->l->use++;
-    n->r = e1.n; n->r->use++;
-  }
-
-  forceinline
-  LinExpr::LinExpr(const LinExpr& e, NodeType t, int c) :
-    n(new Node) {
-    n->n_int = e.n->n_int;
-    n->n_bool = e.n->n_bool;
-    n->t = t;
-    n->l = NULL;
-    n->r = e.n; n->r->use++;
-    n->c = c;
-  }
-
-  forceinline
-  LinExpr::LinExpr(int a, const LinExpr& e) :
-    n(new Node) {
-    n->n_int = e.n->n_int;
-    n->n_bool = e.n->n_bool;
-    n->t = NT_MUL;
-    n->l = e.n; n->l->use++;
-    n->r = NULL;
-    n->a = a;
-  }
-
-  forceinline
-  LinExpr::LinExpr(NonLinExpr* e) :
-    n(new Node) {
-    n->n_int = 1;
-    n->n_bool = 0;
-    n->t = NT_NONLIN;
-    n->l = n->r = NULL;
-    n->a = 0;
-    n->sum.ne = e;
-  }
-
   forceinline int
   LinExpr::Node::fill(Home home, IntConLevel icl,
                       Int::Linear::Term<Int::IntView>* ti, 
