@@ -9,8 +9,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-14 20:32:01 +0200 (Wed, 14 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11195 $
+ *     $Date: 2011-01-21 16:01:14 +0100 (Fri, 21 Jan 2011) $ by $Author: schulte $
+ *     $Revision: 11553 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -60,6 +60,58 @@ namespace Gecode {
   }
 
 
+  SetExpr::SetExpr(const SetVar& x) : n(new Node) {
+    n->same = 1;
+    n->t    = NT_VAR;
+    n->l    = NULL;
+    n->r    = NULL;
+    n->x    = x;
+  }
+
+  SetExpr::SetExpr(const IntSet& s) : n(new Node) {
+    n->same = 1;
+    n->t    = NT_CONST;
+    n->l    = NULL;
+    n->r    = NULL;
+    n->s    = s;
+  }
+
+  SetExpr::SetExpr(const LinExpr& e) : n(new Node) {
+    n->same = 1;
+    n->t    = NT_LEXP;
+    n->l    = NULL;
+    n->r    = NULL;
+    n->e    = e;
+  }
+  
+  SetExpr::SetExpr(const SetExpr& l, NodeType t, const SetExpr& r)
+    : n(new Node) {
+    int ls = same(t,l.n->t) ? l.n->same : 1;
+    int rs = same(t,r.n->t) ? r.n->same : 1;
+    n->same = ls+rs;
+    n->t    = t;
+    n->l    = l.n;
+    n->l->use++;
+    n->r    = r.n;
+    n->r->use++;
+  }
+
+  SetExpr::SetExpr(const SetExpr& l, NodeType t) {
+    (void) t;
+    assert(t == NT_CMPL);
+    if (l.n->t == NT_CMPL) {
+      n = l.n->l;
+      n->use++;
+    } else {
+      n = new Node;
+      n->same = 1;
+      n->t    = NT_CMPL;
+      n->l    = l.n;
+      n->l->use++;
+      n->r    = NULL;
+    }
+  }
+
   const SetExpr&
   SetExpr::operator =(const SetExpr& e) {
     if (this != &e) {
