@@ -13,8 +13,8 @@
  *     Gabor Szokoli, 2004
  *
  *  Last modified:
- *     $Date: 2009-09-08 21:10:29 +0200 (Tue, 08 Sep 2009) $ by $Author: schulte $
- *     $Revision: 9692 $
+ *     $Date: 2011-05-11 12:44:17 +0200 (Wed, 11 May 2011) $ by $Author: tack $
+ *     $Revision: 12001 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -142,13 +142,18 @@ namespace Gecode { namespace Set { namespace Branch {
     return ((a == 0) == inc) ? x.include(home,v) : x.exclude(home,v);
   }
   template<bool inc>
-  forceinline Support::RandomGenerator
+  forceinline typename ValRnd<inc>::Choice
   ValRnd<inc>::choice(Space&) {
     return r;
   }
   template<bool inc>
+  forceinline typename ValRnd<inc>::Choice
+  ValRnd<inc>::choice(const Space&, Archive& e) {
+    return Choice(e.get());
+  }
+  template<bool inc>
   forceinline void
-  ValRnd<inc>::commit(Space&, const Support::RandomGenerator& c,
+  ValRnd<inc>::commit(Space&, const Choice& c,
                       unsigned int) {
     r = c;
   }
