@@ -10,8 +10,8 @@
  *     Guido Tack, 2004, 2005
  *
  *  Last modified:
- *     $Date: 2010-06-07 14:18:14 +0200 (Mon, 07 Jun 2010) $ by $Author: tack $
- *     $Revision: 11048 $
+ *     $Date: 2011-08-24 16:34:16 +0200 (Wed, 24 Aug 2011) $ by $Author: tack $
+ *     $Revision: 12346 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -79,6 +79,38 @@ namespace Gecode {
         }
       }
       break;
+    case SRT_LQ:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::Lq<Set::SetView,Set::ConstSetView,false>
+            ::post(home,s,cv)));
+      }
+      break;
+    case SRT_LE:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::Lq<Set::SetView,Set::ConstSetView,true>
+            ::post(home,s,cv)));
+      }
+      break;
+    case SRT_GQ:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::Lq<Set::ConstSetView,Set::SetView,false>
+            ::post(home,cv,s)));
+      }
+      break;
+    case SRT_GR:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::Lq<Set::ConstSetView,Set::SetView,true>
+            ::post(home,cv,s)));
+      }
+      break;
     case SRT_DISJ:
       {
         if (is.ranges() == 1) {
@@ -171,6 +203,38 @@ namespace Gecode {
                         Set::ConstSetView>::post(home, s, cv, b)));
       }
       break;
+    case SRT_LQ:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::ReLq<Set::SetView,Set::ConstSetView,false>
+            ::post(home,s,cv,b)));
+      }
+      break;
+    case SRT_LE:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::ReLq<Set::SetView,Set::ConstSetView,true>
+            ::post(home,s,cv,b)));
+      }
+      break;
+    case SRT_GQ:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::ReLq<Set::ConstSetView,Set::SetView,false>
+            ::post(home,cv,s,b)));
+      }
+      break;
+    case SRT_GR:
+      {
+        Set::ConstSetView cv(home, is);
+        GECODE_ES_FAIL(
+          (Set::Rel::ReLq<Set::ConstSetView,Set::SetView,true>
+            ::post(home,cv,s,b)));
+      }
+      break;
     case SRT_NQ:
       {
         BoolVar notb(home,0,1);
