@@ -9,8 +9,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-28 16:13:53 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11292 $
+ *     $Date: 2011-08-19 16:47:28 +0200 (Fri, 19 Aug 2011) $ by $Author: tack $
+ *     $Revision: 12318 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -39,39 +39,37 @@
 
 namespace Gecode { namespace Set { namespace Element {
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   forceinline
-  ElementUnion<SView,RView>::
-  ElementUnion(Home home, SView y0,
-                     IdxViewArray& iv0,
-                     RView y1)
-    : Propagator(home), x0(y0), iv(iv0), x1(y1) {
+  ElementUnion<View,View0,View1>::
+  ElementUnion(Home home, IdxViewArray& iv0, View0 y0, View1 y1)
+    : Propagator(home), iv(iv0), x0(y0), x1(y1) {
     home.notice(*this,AP_DISPOSE);
     x0.subscribe(home,*this, PC_SET_ANY);
     x1.subscribe(home,*this, PC_SET_ANY);
     iv.subscribe(home,*this, PC_SET_ANY);
   }
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   forceinline
-  ElementUnion<SView,RView>::
-  ElementUnion(Space& home, bool share,
-                     ElementUnion<SView,RView>& p)
+  ElementUnion<View,View0,View1>::
+  ElementUnion(Space& home, bool share, ElementUnion<View,View0,View1>& p)
     : Propagator(home,share,p) {
     x0.update(home,share,p.x0);
     x1.update(home,share,p.x1);
     iv.update(home,share,p.iv);
   }
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   PropCost
-  ElementUnion<SView,RView>::cost(const Space&, const ModEventDelta&) const {
+  ElementUnion<View,View0,View1>::cost(const Space&,
+                                       const ModEventDelta&) const {
     return PropCost::linear(PropCost::HI, iv.size()+2);
   }
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   forceinline size_t
-  ElementUnion<SView,RView>::dispose(Space& home) {
+  ElementUnion<View,View0,View1>::dispose(Space& home) {
     home.ignore(*this,AP_DISPOSE);
     if (!home.failed()) {
       x0.cancel(home,*this,PC_SET_ANY);
@@ -82,30 +80,29 @@ namespace Gecode { namespace Set { namespace Element {
     return sizeof(*this);
   }
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   ExecStatus
-  ElementUnion<SView,RView>::
-  post(Home home, SView x0, IdxViewArray& xs,
-       RView x1) {
+  ElementUnion<View,View0,View1>::
+  post(Home home, IdxViewArray& xs, View0 x0, View1 x1) {
     int n = xs.size();
 
-    // s2 \subseteq {1,...,n}
+    // x0 \subseteq {1,...,n}
     Iter::Ranges::Singleton s(0, n-1);
-    GECODE_ME_CHECK(x1.intersectI(home,s));
+    GECODE_ME_CHECK(x0.intersectI(home,s));
     (void) new (home)
-      ElementUnion<SView,RView>(home,x0,xs,x1);
+      ElementUnion<View,View0,View1>(home,xs,x0,x1);
     return ES_OK;
   }
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   Actor*
-  ElementUnion<SView,RView>::copy(Space& home, bool share) {
-    return new (home) ElementUnion<SView,RView>(home,share,*this);
+  ElementUnion<View,View0,View1>::copy(Space& home, bool share) {
+    return new (home) ElementUnion<View,View0,View1>(home,share,*this);
   }
 
-  template<class SView, class RView>
+  template<class View, class View0, class View1>
   ExecStatus
-  ElementUnion<SView,RView>::propagate(Space& home, const ModEventDelta&) {
+  ElementUnion<View,View0,View1>::propagate(Space& home, const ModEventDelta&) {
     Region r(home);
     int n = iv.size();
 
@@ -115,19 +112,17 @@ namespace Gecode { namespace Set { namespace Element {
 
       // Cache the upper bound iterator, as we have to
       // modify the upper bound while iterating
-      LubRanges<RView> x1ub(x1);
-      Iter::Ranges::Cache<LubRanges<RView> > x1ubc(r,x1ub);
-      Iter::Ranges::ToValues<Iter::Ranges::Cache<LubRanges<RView> > >
-        vx1ub(x1ubc);
+      LubRanges<View0> x0ub(x0);
+      Iter::Ranges::Cache x0ubc(r,x0ub);
+      Iter::Ranges::ToValues<Iter::Ranges::Cache> vx0ub(x0ubc);
 
-      GlbRanges<RView> x1lb(x1);
-      Iter::Ranges::Cache<GlbRanges<RView> > x1lbc(r,x1lb);
-      Iter::Ranges::ToValues<Iter::Ranges::Cache<GlbRanges<RView> > >
-        vx1(x1lbc);
+      GlbRanges<View0> x0lb(x0);
+      Iter::Ranges::Cache x0lbc(r,x0lb);
+      Iter::Ranges::ToValues<Iter::Ranges::Cache> vx0(x0lbc);
 
       // In the first iteration, compute in before[i] the union
       // of all the upper bounds of the x_i. At the same time,
-      // exclude inconsistent x_i from x1 and remove them from
+      // exclude inconsistent x_i from x0 and remove them from
       // the list, cancel their dependencies.
 
       GLBndSet sofarBefore(home);
@@ -142,33 +137,34 @@ namespace Gecode { namespace Set { namespace Element {
       unsigned int maxCard = 0;
       unsigned int minCard = Limits::card;
 
-      while ( vx1ub() ) {
+      while ( vx0ub() ) {
 
         // Remove vars at indices not in the upper bound
-        if (iv[i].idx < vx1ub.val()) {
+        if (iv[i].idx < vx0ub.val()) {
           iv[i].view.cancel(home,*this, PC_SET_ANY);
           ++i;
           continue;
         }
-        assert(iv[i].idx == vx1ub.val());
+        assert(iv[i].idx == vx0ub.val());
         iv[j] = iv[i];
 
-        SView candidate = iv[j].view;
+        View candidate = iv[j].view;
         int candidateInd = iv[j].idx;
 
-        // inter = glb(candidate) & complement(lub(x0))
-        GlbRanges<SView> candlb(candidate);
-        LubRanges<SView> x0ub(x0);
-        Iter::Ranges::Diff<GlbRanges<SView>,
-          LubRanges<SView> > diff(candlb, x0ub);
+        // inter = glb(candidate) & complement(lub(x1))
+        GlbRanges<View> candlb(candidate);
+        LubRanges<View1> x1ub(x1);
+        Iter::Ranges::Diff<GlbRanges<View>, LubRanges<View1> >
+          diff(candlb, x1ub);
 
         bool selectSingleInconsistent = false;
-        if (x1.cardMax() <= 1) {
-          GlbRanges<SView> x0lb(x0);
-          LubRanges<SView> candub(candidate);
-          Iter::Ranges::Diff<GlbRanges<SView>,
-                             LubRanges<SView> > diff2(x0lb, candub);
-          selectSingleInconsistent = diff2() || candidate.cardMax() < x0.cardMin();
+        if (x0.cardMax() <= 1) {
+          GlbRanges<View1> x1lb(x1);
+          LubRanges<View> candub(candidate);
+          Iter::Ranges::Diff<GlbRanges<View1>,
+                             LubRanges<View> > diff2(x1lb, candub);
+          selectSingleInconsistent =
+            diff2() || candidate.cardMax() < x1.cardMin();
         }
 
         // exclude inconsistent x_i
@@ -176,91 +172,91 @@ namespace Gecode { namespace Set { namespace Element {
         //  * at most one x_i can be selected and there are
         //    elements in x_0 that can't be in x_i
         //    (selectSingleInconsistent)
-        //  * its min cardinality is greater than maxCard of x0
+        //  * its min cardinality is greater than maxCard of x1
         //  * inter is not empty (there are elements in x_i
         //    that can't be in x_0)
         if (selectSingleInconsistent ||
-            candidate.cardMin() > x0.cardMax() ||
+            candidate.cardMin() > x1.cardMax() ||
             diff()) {
-          ModEvent me = (x1.exclude(home,candidateInd));
+          ModEvent me = (x0.exclude(home,candidateInd));
           loopVar |= me_modified(me);
           GECODE_ME_CHECK(me);
 
           iv[j].view.cancel(home,*this, PC_SET_ANY);
           ++i;
-          ++vx1ub;
+          ++vx0ub;
           continue;
         } else {
           // if x_i is consistent, check whether we know
-          // that its index is in x1
-          if (vx1() && vx1.val()==candidateInd) {
-            // x0 >= candidate, candidate <= x0
-            GlbRanges<SView> candlb(candidate);
-            ModEvent me = x0.includeI(home,candlb);
+          // that its index is in x0
+          if (vx0() && vx0.val()==candidateInd) {
+            // x1 >= candidate, candidate <= x1
+            GlbRanges<View> candlb(candidate);
+            ModEvent me = x1.includeI(home,candlb);
             loopVar |= me_modified(me);
             GECODE_ME_CHECK(me);
 
-            LubRanges<SView> x0ub(x0);
-            me = candidate.intersectI(home,x0ub);
+            LubRanges<View1> x1ub(x1);
+            me = candidate.intersectI(home,x1ub);
             loopVar |= me_modified(me);
             GECODE_ME_CHECK(me);
-            ++vx1;
+            ++vx0;
           }
           new (&before[j]) GLBndSet(home);
           before[j].update(home,sofarBefore);
-          LubRanges<SView> cub(candidate);
+          LubRanges<View> cub(candidate);
           sofarBefore.includeI(home,cub);
-          GlbRanges<SView> clb(candidate);
+          GlbRanges<View> clb(candidate);
           selectedInter.intersectI(home,clb);
           maxCard = std::max(maxCard, candidate.cardMax());
           minCard = std::min(minCard, candidate.cardMin());
         }
 
-        ++vx1ub;
+        ++vx0ub;
         ++i; ++j;
       }
 
       // cancel the variables with index greater than
-      // max of lub(x1)
+      // max of lub(x0)
       for (int k=i; k<n; k++) {
         iv[k].view.cancel(home,*this, PC_SET_ANY);
       }
       n = j;
       iv.size(n);
 
-      if (x1.cardMax()==0) {
+      if (x0.cardMax()==0) {
         // Selector is empty, hence the result must be empty
         {
-          GECODE_ME_CHECK(x0.cardMax(home,0));
+          GECODE_ME_CHECK(x1.cardMax(home,0));
         }
         for (int i=n; i--;)
           before[i].dispose(home);
         return home.ES_SUBSUMED(*this);
       }
 
-      if (x1.cardMin() > 0) {
+      if (x0.cardMin() > 0) {
         // Selector is not empty, hence the intersection of the
-        // possibly selected lower bounds is contained in x0
+        // possibly selected lower bounds is contained in x1
         BndSetRanges si(selectedInter);
-        ModEvent me = x0.includeI(home, si);
+        ModEvent me = x1.includeI(home, si);
         loopVar |= me_modified(me);
         GECODE_ME_CHECK(me);
-        me = x0.cardMin(home, minCard);
+        me = x1.cardMin(home, minCard);
         loopVar |= me_modified(me);
         GECODE_ME_CHECK(me);
       }
       selectedInter.dispose(home);
 
-      if (x1.cardMax() <= 1) {
-        ModEvent me = x0.cardMax(home, maxCard);
+      if (x0.cardMax() <= 1) {
+        ModEvent me = x1.cardMax(home, maxCard);
         loopVar |= me_modified(me);
         GECODE_ME_CHECK(me);
       }
 
       {
-        // x0 <= sofarBefore
+        // x1 <= sofarBefore
         BndSetRanges sfB(sofarBefore);
-        ModEvent me = x0.intersectI(home,sfB);
+        ModEvent me = x1.intersectI(home,sfB);
         loopVar |= me_modified(me);
         GECODE_ME_CHECK(me);
       }
@@ -275,15 +271,15 @@ namespace Gecode { namespace Set { namespace Element {
         // TODO: check for size of universe here?
         // if (sofarAfter.size() == 0) break;
 
-        // extra = inter(before[i], sofarAfter) - lub(x0)
+        // extra = inter(before[i], sofarAfter) - lub(x1)
         BndSetRanges b(before[i]);
         BndSetRanges s(sofarAfter);
-        GlbRanges<SView> x0lb(x0);
+        GlbRanges<View1> x1lb(x1);
         Iter::Ranges::Union<BndSetRanges, BndSetRanges> inter(b,s);
-        Iter::Ranges::Diff<GlbRanges<SView>,
-          Iter::Ranges::Union<BndSetRanges,BndSetRanges> > diff(x0lb, inter);
+        Iter::Ranges::Diff<GlbRanges<View1>,
+          Iter::Ranges::Union<BndSetRanges,BndSetRanges> > diff(x1lb, inter);
         if (diff()) {
-          ModEvent me = (x1.include(home,iv[i].idx));
+          ModEvent me = (x0.include(home,iv[i].idx));
           loopVar |= me_modified(me);
           GECODE_ME_CHECK(me);
 
@@ -293,7 +289,7 @@ namespace Gecode { namespace Set { namespace Element {
           GECODE_ME_CHECK(me);
         }
 
-        LubRanges<SView> iviub(iv[i].view);
+        LubRanges<View> iviub(iv[i].view);
         sofarAfter.includeI(home,iviub);
         before[i].dispose(home);
       }
@@ -301,27 +297,28 @@ namespace Gecode { namespace Set { namespace Element {
 
     } while (loopVar);
 
-    // Test whether we determined x1 without determining x0
-    if (x1.assigned() && !x0.assigned()) {
-      int ubsize = static_cast<int>(x1.lubSize());
+    // Test whether we determined x0 without determining x1
+    if (x0.assigned() && !x1.assigned()) {
+      int ubsize = static_cast<int>(x0.lubSize());
       if (ubsize > 2) {
         assert(ubsize==n);
-        ViewArray<SView> is(home,ubsize);
+        ViewArray<View> is(home,ubsize);
         for (int i=n; i--;)
           is[i]=iv[i].view;
-        GECODE_REWRITE(*this,(RelOp::UnionN<SView, SView>
-                        ::post(home(*this),is,x0)));
+        GECODE_REWRITE(*this,(RelOp::UnionN<View, View1>
+                        ::post(home(*this),is,x1)));
       } else if (ubsize == 2) {
         assert(n==2);
-        SView a = iv[0].view;
-        SView b = iv[1].view;
-        GECODE_REWRITE(*this,(RelOp::Union<SView, SView, SView>
-                       ::post(home(*this),a,b,x0)));
+        View a = iv[0].view;
+        View b = iv[1].view;
+        GECODE_REWRITE(*this,(RelOp::Union<View,View,View1>
+                       ::post(home(*this),a,b,x1)));
       } else if (ubsize == 1) {
         assert(n==1);
-        GECODE_REWRITE(*this,(Rel::Eq<SView,SView>::post(home(*this),x0,iv[0].view)));
+        GECODE_REWRITE(*this,
+          (Rel::Eq<View1,View>::post(home(*this),x1,iv[0].view)));
       } else {
-        GECODE_ME_CHECK(x0.cardMax(home, 0));
+        GECODE_ME_CHECK(x1.cardMax(home, 0));
         return home.ES_SUBSUMED(*this);
       }
     }
@@ -333,7 +330,7 @@ namespace Gecode { namespace Set { namespace Element {
         break;
       }
     }
-    if (x0.assigned() && x1.assigned() && allAssigned) {
+    if (x1.assigned() && x0.assigned() && allAssigned) {
       return home.ES_SUBSUMED(*this);
     }
 
