@@ -9,8 +9,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-28 16:13:53 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11292 $
+ *     $Date: 2011-09-19 14:02:26 +0200 (Mon, 19 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12400 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -43,12 +43,15 @@ namespace Gecode { namespace Set { namespace Element {
   forceinline
   ElementUnionConst<SView,RView>::
   ElementUnionConst(Home home, SView y0,
-                     SharedArray<IntSet>& iv0,
-                     RView y1)
-    : Propagator(home), x0(y0), iv(iv0), x1(y1) {
+                    const IntSetArgs& iv0,
+                    RView y1)
+    : Propagator(home), x0(y0), n_iv(iv0.size()), x1(y1) {
     home.notice(*this,AP_DISPOSE);
     x0.subscribe(home,*this, PC_SET_ANY);
     x1.subscribe(home,*this, PC_SET_ANY);
+    iv=static_cast<Space&>(home).alloc<IntSet>(n_iv);
+    for (unsigned int i=iv0.size(); i--;)
+      iv[i]=iv0[i];
   }
 
   template<class SView, class RView>
@@ -56,16 +59,18 @@ namespace Gecode { namespace Set { namespace Element {
   ElementUnionConst<SView,RView>::
   ElementUnionConst(Space& home, bool share,
                      ElementUnionConst<SView,RView>& p)
-    : Propagator(home,share,p) {
+    : Propagator(home,share,p), n_iv(p.n_iv) {
     x0.update(home,share,p.x0);
     x1.update(home,share,p.x1);
-    iv.update(home,share,p.iv);
+    iv=home.alloc<IntSet>(n_iv);
+    for (unsigned int i=n_iv; i--;)
+      iv[i].update(home,share,p.iv[i]);
   }
 
   template<class SView, class RView>
   PropCost
   ElementUnionConst<SView,RView>::cost(const Space&, const ModEventDelta&) const {
-    return PropCost::linear(PropCost::HI, iv.size()+2);
+    return PropCost::linear(PropCost::HI, n_iv+2);
   }
 
   template<class SView, class RView>
@@ -76,7 +81,8 @@ namespace Gecode { namespace Set { namespace Element {
       x0.cancel(home,*this, PC_SET_ANY);
       x1.cancel(home,*this, PC_SET_ANY);
     }
-    iv.~SharedArray();
+    for (unsigned int i=n_iv; i--;)
+      iv[i].~IntSet();
     (void) Propagator::dispose(home);
     return sizeof(*this);
   }
@@ -84,7 +90,7 @@ namespace Gecode { namespace Set { namespace Element {
   template<class SView, class RView>
   ExecStatus
   ElementUnionConst<SView,RView>::
-  post(Home home, SView x0, SharedArray<IntSet>& xs,
+  post(Home home, SView x0, const IntSetArgs& xs,
        RView x1) {
     int n = xs.size();
 
@@ -106,26 +112,25 @@ namespace Gecode { namespace Set { namespace Element {
   ExecStatus
   ElementUnionConst<SView,RView>::propagate(Space& home, const ModEventDelta&) {
     Region r(home);
-    int n = iv.size();
 
-    bool* stillSelected = r.alloc<bool>(n);
+    bool* stillSelected = r.alloc<bool>(n_iv);
 
     bool loopVar;
     do {
       loopVar = false;
-      for (int i=n; i--;)
+      for (int i=n_iv; i--;)
         stillSelected[i] = false;
 
       // Cache the upper bound iterator, as we have to
       // modify the upper bound while iterating
       LubRanges<RView> x1ub(x1);
-      Iter::Ranges::Cache<LubRanges<RView> > x1ubc(r,x1ub);
-      Iter::Ranges::ToValues<Iter::Ranges::Cache<LubRanges<RView> > >
+      Iter::Ranges::Cache x1ubc(r,x1ub);
+      Iter::Ranges::ToValues<Iter::Ranges::Cache>
         vx1ub(x1ubc);
 
       GlbRanges<RView> x1lb(x1);
-      Iter::Ranges::Cache<GlbRanges<RView> > x1lbc(r,x1lb);
-      Iter::Ranges::ToValues<Iter::Ranges::Cache<GlbRanges<RView> > >
+      Iter::Ranges::Cache x1lbc(r,x1lb);
+      Iter::Ranges::ToValues<Iter::Ranges::Cache>
         vx1(x1lbc);
 
       // In the first iteration, compute in before[i] the union
@@ -136,16 +141,14 @@ namespace Gecode { namespace Set { namespace Element {
       LUBndSet selectedInter(home, IntSet (Limits::min,
                                    Limits::max));
       GLBndSet* before =
-        static_cast<GLBndSet*>(r.ralloc(sizeof(GLBndSet)*n));
-
-      int i = 0;
+        static_cast<GLBndSet*>(r.ralloc(sizeof(GLBndSet)*n_iv));
 
       unsigned int maxCard = 0;
       unsigned int minCard = Limits::card;
 
-      while ( vx1ub() ) {
+      while (vx1ub()) {
+        int i = vx1ub.val();
 
-        i = vx1ub.val();
         IntSetRanges candCardR(iv[i]);
         unsigned int candidateCard = Iter::Ranges::size(candCardR);
 
@@ -208,7 +211,7 @@ namespace Gecode { namespace Set { namespace Element {
         {
           GECODE_ME_CHECK(x0.cardMax(home,0));
         }
-        for (int i=n; i--;)
+        for (int i=n_iv; i--;)
           if (stillSelected[i])
             before[i].dispose(home);
         selectedInter.dispose(home);
@@ -249,7 +252,7 @@ namespace Gecode { namespace Set { namespace Element {
 
       // In the second iteration, this time backwards, compute
       // sofarAfter as the union of all lub(x_j) with j>i
-      for (int i=n; i--;) {
+      for (int i=n_iv; i--;) {
         if (!stillSelected[i])
           continue;
         BndSetRanges b(before[i]);
@@ -277,9 +280,8 @@ namespace Gecode { namespace Set { namespace Element {
 
     } while (loopVar);
 
-    if (x0.assigned() || x1.assigned()) {
+    if (x0.assigned() || x1.assigned())
       return home.ES_SUBSUMED(*this);
-    }
 
     return ES_FIX;
   }
