@@ -13,8 +13,8 @@
  *     Gabor Szokoli, 2004
  *
  *  Last modified:
- *     $Date: 2010-09-03 16:36:31 +0200 (Fri, 03 Sep 2010) $ by $Author: schulte $
- *     $Revision: 11390 $
+ *     $Date: 2011-08-08 22:41:45 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12255 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -578,13 +578,10 @@ namespace Set { namespace RelOp {
       }
     }
     if (nonEmptyCounter !=0) {
-      Iter::Ranges::NaryUnion<GlbRanges<View0> >
-        xLBUnion(r,xLBs,nonEmptyCounter);
+      Iter::Ranges::NaryUnion xLBUnion(r,xLBs,nonEmptyCounter);
       BndSetRanges dets(unionOfDets);
-      Iter::Ranges::Union<Iter::Ranges::NaryUnion<GlbRanges<View0> >,
-        BndSetRanges>
-        allUnion(xLBUnion,dets);
-      GECODE_ME_CHECK_MODIFIED(modified, y.includeI(home,allUnion));
+      xLBUnion |= dets;
+      GECODE_ME_CHECK_MODIFIED(modified, y.includeI(home,xLBUnion));
     }
     return ES_FIX;
   }
@@ -606,14 +603,11 @@ namespace Set { namespace RelOp {
         nonEmptyCounter++;
       }
     }
-    if (nonEmptyCounter !=0) {
-      Iter::Ranges::NaryUnion<LubRanges<View0> >
-        xUBUnion(r,xUBs,nonEmptyCounter);
+    if (nonEmptyCounter != 0) {
+      Iter::Ranges::NaryUnion xUBUnion(r,xUBs,nonEmptyCounter);
       BndSetRanges dets(unionOfDets);
-      Iter::Ranges::Union<Iter::Ranges::NaryUnion<LubRanges<View0> >,
-        BndSetRanges>
-        fullUnion(xUBUnion, dets);
-      GECODE_ME_CHECK_MODIFIED(modified, y.intersectI(home,fullUnion));
+      xUBUnion |= dets;
+      GECODE_ME_CHECK_MODIFIED(modified, y.intersectI(home,xUBUnion));
     }
     return ES_FIX;
   }
