@@ -13,8 +13,8 @@
  *     Gabor Szokoli, 2004
  *
  *  Last modified:
- *     $Date: 2010-07-28 16:13:53 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11292 $
+ *     $Date: 2011-08-08 18:04:53 +0200 (Mon, 08 Aug 2011) $ by $Author: schulte $
+ *     $Revision: 12253 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -79,8 +79,7 @@ namespace Gecode { namespace Set { namespace Rel {
       GlbRanges<View0> x0lb(x0);
       GlbRanges<View1> x1lb(x1);
       Iter::Ranges::Union<GlbRanges<View0>,GlbRanges<View1> > lbu(x0lb,x1lb);
-      Iter::Ranges::Cache<Iter::Ranges::Union
-        <GlbRanges<View0>, GlbRanges<View1> > > lbuc(r,lbu);
+      Iter::Ranges::Cache lbuc(r,lbu);
       GECODE_ME_CHECK(x0.includeI(home,lbuc));
       lbuc.reset();
       GECODE_ME_CHECK(x1.includeI(home,lbuc));
@@ -90,8 +89,7 @@ namespace Gecode { namespace Set { namespace Rel {
       LubRanges<View0> x0ub(x0);
       LubRanges<View1> x1ub(x1);
       Iter::Ranges::Inter<LubRanges<View0>,LubRanges<View1> > ubi(x0ub,x1ub);
-      Iter::Ranges::Cache<Iter::Ranges::Inter
-        <LubRanges<View0>,LubRanges<View1> > > ubic(r,ubi);
+      Iter::Ranges::Cache ubic(r,ubi);
       GECODE_ME_CHECK(x0.intersectI(home,ubic));
       ubic.reset();
       GECODE_ME_CHECK(x1.intersectI(home,ubic));
