@@ -13,8 +13,8 @@
  *     Gabor Szokoli, 2004
  *
  *  Last modified:
- *     $Date: 2010-03-03 17:32:21 +0100 (Wed, 03 Mar 2010) $ by $Author: schulte $
- *     $Revision: 10364 $
+ *     $Date: 2011-05-23 16:48:31 +0200 (Mon, 23 May 2011) $ by $Author: tack $
+ *     $Revision: 12018 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -120,17 +120,17 @@ namespace Gecode { namespace Set { namespace Sequence {
       modified = false;
 
       if (oldModified || modified || anybevent || cardevent)
-        GECODE_ME_CHECK(propagateSeq(home,modified,assigned,x));
+        GECODE_ES_CHECK(propagateSeq(home,modified,assigned,x));
       if (oldModified || modified || anybevent)
-        GECODE_ME_CHECK(propagateSeqUnion(home,modified,x,y));
+        GECODE_ES_CHECK(propagateSeqUnion(home,modified,x,y));
       if (oldModified || modified || ubevent)
-        GECODE_ME_CHECK(RelOp::unionNXiUB(home,modified,x,y,unionOfDets));
+        GECODE_ES_CHECK(RelOp::unionNXiUB(home,modified,x,y,unionOfDets));
       if (oldModified || modified || ubevent)
-        GECODE_ME_CHECK(RelOp::partitionNYUB(home,modified,x,y,unionOfDets));
+        GECODE_ES_CHECK(RelOp::partitionNYUB(home,modified,x,y,unionOfDets));
       if (oldModified || modified || anybevent)
-        GECODE_ME_CHECK(RelOp::partitionNXiLB(home,modified,x,y,unionOfDets));
+        GECODE_ES_CHECK(RelOp::partitionNXiLB(home,modified,x,y,unionOfDets));
       if (oldModified || modified || cardevent || ubevent)
-        GECODE_ME_CHECK(RelOp::partitionNCard(home,modified,x,y,unionOfDets));
+        GECODE_ES_CHECK(RelOp::partitionNCard(home,modified,x,y,unionOfDets));
 
     } while (modified);
 
