@@ -7,8 +7,8 @@
  *     Christian Schulte, 2008
  *
  *  Last modified:
- *     $Date: 2010-04-28 18:54:51 +0200 (Wed, 28 Apr 2010) $ by $Author: tack $
- *     $Revision: 10822 $
+ *     $Date: 2010-07-26 12:53:58 +0200 (Mon, 26 Jul 2010) $ by $Author: schulte $
+ *     $Revision: 11279 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -296,6 +296,8 @@ namespace Gecode {
     void* ralloc(size_t s);
     /// Free memory block starting at \a p
     void  rfree(void* p);
+    /// Free memory block starting at \a p with size \a s
+    void  rfree(void* p, size_t s);
     /// Change memory block starting at \a p to size \a s
     void* rrealloc(void* p, size_t s);
     //@}
@@ -330,6 +332,11 @@ namespace Gecode {
     ::free(p);
   }
 
+  forceinline void
+  Heap::rfree(void* p, size_t) {
+    ::free(p);
+  }
+
   forceinline void*
   Heap::rrealloc(void* p, size_t s) {
     p = ::realloc(p,s);
