@@ -7,8 +7,8 @@
  *     Christian Schulte, 2009
  *
  *  Last modified:
- *     $Date: 2009-09-07 11:03:59 +0200 (Mon, 07 Sep 2009) $ by $Author: schulte $
- *     $Revision: 9665 $
+ *     $Date: 2011-04-28 14:05:18 +0200 (Thu, 28 Apr 2011) $ by $Author: tack $
+ *     $Revision: 11968 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -130,7 +130,7 @@ namespace Gecode { namespace Support {
   forceinline unsigned int
   Thread::npu(void) {
 #ifdef GECODE_HAS_UNISTD_H
-    int n=sysconf(_SC_NPROCESSORS_ONLN);
+    int n=static_cast<int>(sysconf(_SC_NPROCESSORS_ONLN));
     return (n>1) ? n : 1;
 #else
     return 1;
