@@ -9,8 +9,8 @@
  *     Mikael Lagerkvist, 2006
  *
  *  Last modified:
- *     $Date: 2009-08-26 13:18:40 +0200 (Wed, 26 Aug 2009) $ by $Author: schulte $
- *     $Revision: 9624 $
+ *     $Date: 2010-06-03 13:11:11 +0200 (Thu, 03 Jun 2010) $ by $Author: tack $
+ *     $Revision: 11013 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -109,7 +109,44 @@ namespace Test { namespace Int {
     delete [] vals;
   }
 
+  forceinline int
+  RandomMixAssignment::randval(const Gecode::IntSet& d) {
+    unsigned int skip = Base::rand(d.size());
+    for (Gecode::IntSetRanges it(d); true; ++it) {
+      if (it.width() > skip)
+        return it.min() + static_cast<int>(skip);
+      skip -= it.width();
+    }
+    GECODE_NEVER;
+    return 0;
+  }
 
+  inline
+  RandomMixAssignment::RandomMixAssignment(int n0, const Gecode::IntSet& d0,
+                                           int n1, const Gecode::IntSet& d1, 
+                                           int a0)
+    : Assignment(n0+n1,d0),vals(new int[n0+n1]),a(a0),_n1(n1),_d1(d1) {
+    for (int i=n0; i--; )
+      vals[i] = randval(d);
+    for (int i=n1; i--; )
+      vals[n0+i] = randval(_d1);
+  }
+
+  inline bool
+  RandomMixAssignment::operator()(void) const {
+    return a>0;
+  }
+
+  inline int
+  RandomMixAssignment::operator[](int i) const {
+    assert((i>=0) && (i<n));
+    return vals[i];
+  }
+
+  inline
+  RandomMixAssignment::~RandomMixAssignment(void) {
+    delete [] vals;
+  }
 
   /*
    * Tests with integer constraints
@@ -120,14 +157,14 @@ namespace Test { namespace Int {
              bool r, Gecode::IntConLevel i)
     : Base("Int::"+s), arity(a), dom(d), reified(r), icl(i),
       contest(icl == Gecode::ICL_DOM ? CTL_DOMAIN : CTL_NONE),
-      testsearch(true) {}
+      testsearch(true), testfix(true) {}
 
   inline
   Test::Test(const std::string& s, int a, int min, int max,
              bool r, Gecode::IntConLevel i)
     : Base("Int::"+s), arity(a), dom(min,max), reified(r), icl(i),
       contest(icl == Gecode::ICL_DOM ? CTL_DOMAIN : CTL_NONE),
-      testsearch(true) {}
+      testsearch(true), testfix(true) {}
 
   inline
   std::string
