@@ -9,8 +9,8 @@
  *     Mikael Lagerkvist, 2006
  *
  *  Last modified:
- *     $Date: 2010-06-04 16:13:00 +0200 (Fri, 04 Jun 2010) $ by $Author: schulte $
- *     $Revision: 11028 $
+ *     $Date: 2011-09-07 16:15:16 +0200 (Wed, 07 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12394 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -57,12 +57,12 @@ namespace Test { namespace Int {
      public:
        /// Create and register test
        Distinct(const Gecode::IntSet& d0, Gecode::IntConLevel icl)
-         : Test(std::string(useCount ? "Count::Distinct" : "Distinct")+
-                "::Sparse::"+str(icl),6,d0,false,icl), d(d0) {}
+         : Test(std::string(useCount ? "Count::Distinct::" : "Distinct::")+
+                str(icl)+"::Sparse",6,d0,false,icl), d(d0) {}
        /// Create and register test
        Distinct(int min, int max, Gecode::IntConLevel icl)
-         : Test(std::string(useCount ? "Count::Distinct" : "Distinct")+
-                "::Dense::"+str(icl),6,min,max,false,icl), d(min,max) {}
+         : Test(std::string(useCount ? "Count::Distinct::" : "Distinct::")+
+                str(icl)+"::Dense",6,min,max,false,icl), d(min,max) {}
        /// Check whether \a x is solution
        virtual bool solution(const Assignment& x) const {
          for (int i=0; i<x.size(); i++)
