@@ -7,8 +7,8 @@
  *     Christian Schulte, 2005
  *
  *  Last modified:
- *     $Date: 2010-04-08 12:35:31 +0200 (Thu, 08 Apr 2010) $ by $Author: schulte $
- *     $Revision: 10684 $
+ *     $Date: 2010-08-23 18:53:25 +0200 (Mon, 23 Aug 2010) $ by $Author: tack $
+ *     $Revision: 11358 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -102,6 +102,26 @@ namespace Test { namespace Int {
        }
      };
 
+     /// %Test for domain constraint (empty range)
+     class DomRangeEmpty : public Test {
+     public:
+       /// Create and register test
+       DomRangeEmpty(void) : Test("Dom::Range::Empty",1,-4,4,true) {}
+       /// %Test whether \a x is solution
+       virtual bool solution(const Assignment&) const {
+         return false;
+       }
+       /// Post constraint on \a x
+       virtual void post(Gecode::Space& home, Gecode::IntVarArray&) {
+         home.fail();
+       }
+       /// Post reified constraint on \a x for \a b
+       virtual void post(Gecode::Space& home, Gecode::IntVarArray& x,
+                         Gecode::BoolVar b) {
+         Gecode::dom(home, x[0], 3, 2, b);
+       }
+     };
+
 
      const int r[4][2] = {
        {-4,-3},{-1,-1},{1,1},{3,5}
@@ -144,6 +164,7 @@ namespace Test { namespace Int {
      DomRange dr3(3);
      DomDom dd1(1);
      DomDom dd3(3);
+     DomRangeEmpty dre;
      //@}
 
    }
