@@ -7,8 +7,8 @@
  *     Christian Schulte, 2005
  *
  *  Last modified:
- *     $Date: 2010-04-08 12:35:31 +0200 (Thu, 08 Apr 2010) $ by $Author: schulte $
- *     $Revision: 10684 $
+ *     $Date: 2011-07-06 21:56:28 +0200 (Wed, 06 Jul 2011) $ by $Author: schulte $
+ *     $Revision: 12151 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -261,24 +261,32 @@ namespace Test { namespace Int {
        }
      };
 
-     /// %Test for pairwise relation between integer variables
-     class IntPairwise : public Test {
+     /// %Test for sequence of relations between integer variables
+     class IntSeq : public Test {
      protected:
        /// Integer relation type to propagate
        Gecode::IntRelType irt;
      public:
        /// Create and register test
-       IntPairwise(Gecode::IntRelType irt0, Gecode::IntConLevel icl)
-         : Test("Rel::Int::Pairwise::"+str(irt0)+"::"+str(icl),
-                   4,-3,3,false,icl),
+       IntSeq(int n, Gecode::IntRelType irt0, Gecode::IntConLevel icl)
+         : Test("Rel::Int::Seq::"+str(n)+"::"+str(irt0)+"::"+str(icl),
+                n,-3,3,false,icl),
            irt(irt0) {}
        /// %Test whether \a x is solution
        virtual bool solution(const Assignment& x) const {
-         for (int i=0; i<x.size(); i++)
-           for (int j=i+1; j<x.size(); j++)
-             if (!cmp(x[i],irt,x[j]))
+         if (irt == Gecode::IRT_NQ) {
+           if (x.size() < 2)
+             return false;
+           for (int i=0; i<x.size()-1; i++)
+             if (x[i] != x[i+1])
+               return true;
+           return false;
+         } else {
+           for (int i=0; i<x.size()-1; i++)
+             if (!cmp(x[i],irt,x[i+1]))
                return false;
-         return true;
+           return true;
+         }
        }
        /// Post constraint on \a x
        virtual void post(Gecode::Space& home, Gecode::IntVarArray& x) {
@@ -286,23 +294,70 @@ namespace Test { namespace Int {
        }
      };
 
-     /// %Test for pairwise relation between Boolean variables
-     class BoolPairwise : public Test {
+     /// %Test for sequence of relations between shared integer variables
+     class IntSharedSeq : public Test {
      protected:
        /// Integer relation type to propagate
        Gecode::IntRelType irt;
      public:
        /// Create and register test
-       BoolPairwise(Gecode::IntRelType irt0)
-         : Test("Rel::Bool::Pairwise::"+str(irt0),8,0,1),
+       IntSharedSeq(int n, Gecode::IntRelType irt0, Gecode::IntConLevel icl)
+         : Test("Rel::Int::Seq::Shared::"+str(n)+"::"+str(irt0)+"::"+str(icl),
+                n,-3,3,false,icl),
            irt(irt0) {}
        /// %Test whether \a x is solution
        virtual bool solution(const Assignment& x) const {
-         for (int i=0; i<x.size(); i++)
-           for (int j=i+1; j<x.size(); j++)
-             if (!cmp(x[i],irt,x[j]))
+         if (irt == Gecode::IRT_NQ) {
+           if (x.size() < 2)
+             return false;
+           for (int i=0; i<x.size()-1; i++)
+             if (x[i] != x[i+1])
+               return true;
+           return false;
+         } else {
+           int n = x.size();
+           for (int i=0; i<2*n-1; i++)
+             if (!cmp(x[i % n],irt,x[(i+1) % n]))
                return false;
-         return true;
+           return true;
+         }
+       }
+       /// Post constraint on \a x
+       virtual void post(Gecode::Space& home, Gecode::IntVarArray& x) {
+         using namespace Gecode;
+         int n = x.size();
+         IntVarArgs y(2*n);
+         for (int i=n; i--; ) 
+           y[i] = y[n+i] = x[i];
+         rel(home, y, irt, icl);
+       }
+     };
+
+     /// %Test for sequence of relations between Boolean variables
+     class BoolSeq : public Test {
+     protected:
+       /// Integer relation type to propagate
+       Gecode::IntRelType irt;
+     public:
+       /// Create and register test
+       BoolSeq(int n, Gecode::IntRelType irt0)
+         : Test("Rel::Bool::Seq::"+str(n)+"::"+str(irt0),n,0,1),
+           irt(irt0) {}
+       /// %Test whether \a x is solution
+       virtual bool solution(const Assignment& x) const {
+         if (irt == Gecode::IRT_NQ) {
+           if (x.size() < 2)
+             return false;
+           for (int i=0; i<x.size()-1; i++)
+             if (x[i] != x[i+1])
+               return true;
+           return false;
+         } else {
+           for (int i=0; i<x.size()-1; i++)
+             if (!cmp(x[i],irt,x[i+1]))
+               return false;
+           return true;
+         }
        }
        /// Post constraint on \a x
        virtual void post(Gecode::Space& home, Gecode::IntVarArray& x) {
@@ -314,7 +369,45 @@ namespace Test { namespace Int {
        }
      };
 
-     /// %Test for relation between arrays of integer variables
+     /// %Test for sequence of relations between shared Boolean variables
+     class BoolSharedSeq : public Test {
+     protected:
+       /// Integer relation type to propagate
+       Gecode::IntRelType irt;
+     public:
+       /// Create and register test
+       BoolSharedSeq(int n, Gecode::IntRelType irt0)
+         : Test("Rel::Bool::Seq::Shared::"+str(n)+"::"+str(irt0),n,0,1),
+           irt(irt0) {}
+       /// %Test whether \a x is solution
+       virtual bool solution(const Assignment& x) const {
+         if (irt == Gecode::IRT_NQ) {
+           if (x.size() < 2)
+             return false;
+           for (int i=0; i<x.size()-1; i++)
+             if (x[i] != x[i+1])
+               return true;
+           return false;
+         } else {
+           int n = x.size();
+           for (int i=0; i<2*n-1; i++)
+             if (!cmp(x[i % n],irt,x[(i+1) % n]))
+               return false;
+           return true;
+         }
+       }
+       /// Post constraint on \a x
+       virtual void post(Gecode::Space& home, Gecode::IntVarArray& x) {
+         using namespace Gecode;
+         int n = x.size();
+         BoolVarArgs b(2*n);
+         for (int i=n; i--; )
+           b[i]=b[n+i]=channel(home,x[i]);
+         rel(home, b, irt);
+       }
+     };
+
+     /// %Test for relation between same sized arrays of integer variables
      class IntArray : public Test {
      protected:
        /// Integer relation type to propagate
@@ -346,6 +439,46 @@ namespace Test { namespace Int {
        }
      };
 
+     /// %Test for relation between differently sized arrays of integer variables
+     class IntArrayDiff : public Test {
+     protected:
+       /// Integer relation type to propagate
+       Gecode::IntRelType irt;
+       /// How many variables in total
+       static const int n = 4;
+       /// How big is the first array
+       int n_fst;
+     public:
+       /// Create and register test
+       IntArrayDiff(Gecode::IntRelType irt0, int m)
+         : Test("Rel::Int::Array::"+str(irt0)+"::"+str(m)+"::"+str(n-m),
+                n,-2,2), 
+           irt(irt0), n_fst(m) {
+         assert(n_fst <= n);
+       }
+       /// %Test whether \a x is solution
+       virtual bool solution(const Assignment& x) const {
+         int n_snd = n - n_fst;
+         for (int i=0; i<std::min(n_fst,n_snd); i++)
+           if (x[i] != x[n_fst+i])
+             return cmp(x[i],irt,x[n_fst+i]);
+         return cmp(n_fst,irt,n_snd);
+       }
+       /// Post constraint on \a x
+       virtual void post(Gecode::Space& home, Gecode::IntVarArray& x) {
+         using namespace Gecode;
+         int n_snd = n - n_fst;
+         IntVarArgs y(n_fst); IntVarArgs z(n_snd);
+         for (int i=0; i<n_fst; i++) {
+           y[i]=x[i];
+         }
+         for (int i=0; i<n_snd; i++) {
+           z[i]=x[n_fst + i];
+         }
+         rel(home, y, irt, z);
+       }
+     };
+
      /// %Test for relation between arrays of Boolean variables
      class BoolArray : public Test {
      protected:
@@ -389,12 +522,27 @@ namespace Test { namespace Int {
              (void) new IntVarXY(irts.irt(),1,icls.icl());
              (void) new IntVarXY(irts.irt(),2,icls.icl());
              (void) new IntVarXX(irts.irt(),icls.icl());
-             (void) new IntPairwise(irts.irt(),icls.icl());
+             (void) new IntSeq(1,irts.irt(),icls.icl());
+             (void) new IntSeq(2,irts.irt(),icls.icl());
+             (void) new IntSeq(3,irts.irt(),icls.icl());
+             (void) new IntSeq(5,irts.irt(),icls.icl());
+             (void) new IntSharedSeq(1,irts.irt(),icls.icl());
+             (void) new IntSharedSeq(2,irts.irt(),icls.icl());
+             (void) new IntSharedSeq(3,irts.irt(),icls.icl());
+             (void) new IntSharedSeq(4,irts.irt(),icls.icl());
            }
            (void) new BoolVarXY(irts.irt(),1);
            (void) new BoolVarXY(irts.irt(),2);
            (void) new BoolVarXX(irts.irt());
-           (void) new BoolPairwise(irts.irt());
+           (void) new BoolSeq(1,irts.irt());
+           (void) new BoolSeq(2,irts.irt());
+           (void) new BoolSeq(3,irts.irt());
+           (void) new BoolSeq(10,irts.irt());
+           (void) new BoolSharedSeq(1,irts.irt());
+           (void) new BoolSharedSeq(2,irts.irt());
+           (void) new BoolSharedSeq(3,irts.irt());
+           (void) new BoolSharedSeq(4,irts.irt());
+           (void) new BoolSharedSeq(8,irts.irt());
            for (int c=-4; c<=4; c++) {
              (void) new IntInt(irts.irt(),1,c);
              (void) new IntInt(irts.irt(),2,c);
@@ -404,6 +552,8 @@ namespace Test { namespace Int {
              (void) new BoolInt(irts.irt(),2,c);
            }
            (void) new IntArray(irts.irt());
+           for (int n_fst=0; n_fst<=4; n_fst++)
+             (void) new IntArrayDiff(irts.irt(),n_fst);
            (void) new BoolArray(irts.irt());
          }
        }
