@@ -11,8 +11,8 @@
  *     Mikael Lagerkvist, 2005
  *
  *  Last modified:
- *     $Date: 2009-05-14 19:35:46 +0200 (Thu, 14 May 2009) $ by $Author: tack $
- *     $Revision: 9118 $
+ *     $Date: 2011-08-24 16:34:16 +0200 (Wed, 24 Aug 2011) $ by $Author: tack $
+ *     $Revision: 12346 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -180,9 +180,13 @@ namespace Test { namespace Set {
       olog << ind(4) << "x[" << i << "] ";
       switch (srt) {
       case Gecode::SRT_EQ: olog << "="; break;
+      case Gecode::SRT_LQ: olog << "<="; break;
+      case Gecode::SRT_LE: olog << "<"; break;
+      case Gecode::SRT_GQ: olog << ">="; break;
+      case Gecode::SRT_GR: olog << ">"; break;
       case Gecode::SRT_NQ: olog << "!="; break;
-      case Gecode::SRT_SUB: olog << "<="; break;
-      case Gecode::SRT_SUP: olog << ">="; break;
+      case Gecode::SRT_SUB: olog << "sub"; break;
+      case Gecode::SRT_SUP: olog << "sup"; break;
       case Gecode::SRT_DISJ: olog << "||"; break;
       case Gecode::SRT_CMPL: olog << "^-1 = "; break;
       }
