@@ -7,8 +7,8 @@
  *     Guido Tack, 2005
  *
  *  Last modified:
- *     $Date: 2010-07-28 18:45:22 +0200 (Wed, 28 Jul 2010) $ by $Author: schulte $
- *     $Revision: 11297 $
+ *     $Date: 2011-02-22 10:26:48 +0100 (Tue, 22 Feb 2011) $ by $Author: tack $
+ *     $Revision: 11761 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -446,7 +446,7 @@ namespace Test { namespace Set {
       }
       /// Post constraint on \a x
       virtual void post(Space& home, SetVarArray& x, IntVarArray& y) {
-        Gecode::channel(home, y, x[0]);
+        Gecode::channelSorted(home, y, x[0]);
       }
     };
     Match _match("Int::Match");
