@@ -7,8 +7,8 @@
  *     Guido Tack, 2005
  *
  *  Last modified:
- *     $Date: 2010-09-03 16:36:31 +0200 (Fri, 03 Sep 2010) $ by $Author: schulte $
- *     $Revision: 11390 $
+ *     $Date: 2011-09-01 15:04:29 +0200 (Thu, 01 Sep 2011) $ by $Author: schulte $
+ *     $Revision: 12384 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -91,7 +91,7 @@ namespace Test { namespace Set {
         , sot(sot0), srt(srt0), share(share0) {}
       /// %Test whether \a x is solution
       bool solution(const SetAssignment& x) const {
-        int a,b,c;
+        int a=0, b=0, c=0;
         switch (share) {
           case 0: a=x[0]; b=x[1]; c=x[2]; break;
           case 1: a=x[0]; b=x[0]; c=x[0]; break;
@@ -253,41 +253,53 @@ namespace Test { namespace Set {
             bool eq;
             if (withConst) {
               Region r(*fs);
-              Iter::Ranges::NaryUnion<CountableSetRanges> u(r, isrs, realN);
+              Iter::Ranges::NaryUnion u(r, isrs, realN);
               IntSetRanges isr(is);
               Iter::Ranges::Union<IntSetRanges,
-                Iter::Ranges::NaryUnion<CountableSetRanges> > uu(isr, u);
+                Iter::Ranges::NaryUnion> uu(isr, u);
               eq = Iter::Ranges::equal(uu, xnr);
             } else {
               Region r(*fs);
-              Iter::Ranges::NaryUnion<CountableSetRanges> u(r, isrs, realN);
+              Iter::Ranges::NaryUnion u(r, isrs, realN);
               eq = Iter::Ranges::equal(u, xnr);
             }
-            delete[] isrs;
+            delete [] isrs;
             delete fs;
             return eq;
           }
         case SOT_INTER:
           {
             if (withConst) {
-              Iter::Ranges::NaryInter<CountableSetRanges> u(isrs, realN);
-              IntSetRanges isr(is);
-              Iter::Ranges::Inter<IntSetRanges,
-                Iter::Ranges::NaryInter<CountableSetRanges> > uu(isr, u);
-              bool eq = (realN == 0 ? Iter::Ranges::equal(isr, xnr) :
-                                      Iter::Ranges::equal(uu, xnr));
-              delete[] isrs;
+              FakeSpace* fs = new FakeSpace;
+              bool eq;
+              {
+                Region r(*fs);
+                Iter::Ranges::NaryInter u(r, isrs, realN);
+                IntSetRanges isr(is);
+                Iter::Ranges::Inter<IntSetRanges,
+                  Iter::Ranges::NaryInter> uu(isr, u);
+                eq = (realN == 0 ? Iter::Ranges::equal(isr, xnr) :
+                           Iter::Ranges::equal(uu, xnr));
+                delete [] isrs;
+              }
+              delete fs;
               return eq;
             } else {
               if (realN == 0) {
                 bool ret =
                   Iter::Ranges::size(xnr) ==  Gecode::Set::Limits::card;
-                delete[] isrs;
+                delete [] isrs;
                 return ret;
               } else {
-                Iter::Ranges::NaryInter<CountableSetRanges> u(isrs, realN);
-                bool eq = Iter::Ranges::equal(u, xnr);
-                delete[] isrs;
+                FakeSpace* fs = new FakeSpace;
+                bool eq;
+                {
+                  Region r(*fs);
+                  Iter::Ranges::NaryInter u(r,isrs, realN);
+                  eq = Iter::Ranges::equal(u, xnr);
+                }
+                delete [] isrs;
+                delete fs;
                 return eq;
               }
             }
