@@ -7,8 +7,8 @@
  *     Guido Tack, 2007
  *
  *  Last modified:
- *     $Date: 2010-04-07 17:28:47 +0200 (Wed, 07 Apr 2010) $ by $Author: tack $
- *     $Revision: 10676 $
+ *     $Date: 2011-09-21 11:18:38 +0200 (Wed, 21 Sep 2011) $ by $Author: tack $
+ *     $Revision: 12410 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -67,9 +67,21 @@ int main(int argc, char** argv) {
   }
 
   if (fg) {
-    fg->createBranchers(fg->solveAnnotations(), false, std::cerr);
+    
+    fg->createBranchers(fg->solveAnnotations(), opt.seed(), false, std::cerr);
     fg->shrinkArrays(p);
-    fg->run(std::cout, p, opt, t_total);
+    if (opt.output()) {
+      std::ofstream os(opt.output());
+      if (!os.good()) {
+        std::cerr << "Could not open file " << opt.output() << " for output."
+                  << std::endl;
+        exit(EXIT_FAILURE);
+      }      
+      fg->run(os, p, opt, t_total);
+      os.close();
+    } else {
+      fg->run(std::cout, p, opt, t_total);
+    }
   } else {
     exit(EXIT_FAILURE);    
   }
